<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Http\Controllers\OldReportController;
use Illuminate\Contracts\Session\Session as SessionSession;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Session;
use PDF;
use App\Http\Controllers\PdfDownloadController;
use App\Exports\ObserverDetailReportsExport;
use App\Exports\CityBillExportReport;
use App\Exports\CityBillUnsubmittedExportReport;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Facades\Excel;
use App\Services\MasterServices;
use Illuminate\Support\Facades\Storage;
use Carbon\Carbon;


class AdminDashboardController extends Controller
{
    public function index()
    {
        $examMode = config('constants.Exam_Mode', []);
        $title = 'Dashboard';
        return view('admin.dashboard', ['examMode' => $examMode,'title'=>$title]);
    }

    public function superIndex()
    {
        $examMode = config('constants.Exam_Mode', []);
        $title = 'SuperAdmin Dashboard';
        return view('admin.superdashboard', ['examMode' => $examMode,'title'=>$title]);
    }

    public function serachExamByExamMode(Request $request)
    {
        $examMode = $request->post('examMode');
        $examTable = config('constants.ExamDates', []);
        $exams = [];
        $exams = DB::select(DB::raw("exec getCommonDataSP :select, :input, :table, :orderBy"), [
            'select' => '[Exam Type Id] as Exam_Type_Id,[Exam Name] as Exam_Name,[Exam Type] as ExamType',
            'input' => "WHERE Exam_Mode =  '{$examMode}' AND Is_Active=1",
            'table' => "[$examTable]",
            'orderBy' => 'ORDER BY [Exam Name] ASC'
        ]);

        return response()->json([
            'success' => true,
            'data' => $exams,
        ]);
    }

    public function getTotalCountBill(Request $request)
    {

        $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
        $TABillHeaderTbl = config('constants.TABillHeaderTbl');
        $CentreBillTbl = config('constants.CentreBillTbl');
        $ExamProfile = config('constants.ExamProfile');
        $examType = $request->post('examType');
        

        $tables = [
            $TABillHeaderTbl => ['role' => [5, 2, 3, 10],'table'=>'OBSERVER',  'type' => 'OBSERVER BILL', 'exam_type_column' => 'Exam Code'], // observer table
            $CentreBillTbl => ['role' => [14], 'table'=>'CENTRE','type' => 'CENTRE SUPERINTENDENT BILL', 'exam_type_column' => 'Exam Type'], // Centre Bill table
            $CityCoordinatorOfflineTbl => ['role' => [1], 'table'=>'CITY','type' => 'CITY COORDINATOR BILL', 'exam_type_column' => 'Exam Type'], // City Coordinator Bill table
        ];
        $data = [];
        foreach ($tables as $tableName => $info) {


            if($info['table'] == 'CENTRE'){
                
                $examProfileCountsAll = DB::table($ExamProfile)
                ->selectRaw(' count(*) as totalBill')
                ->where('ExamType', $examType)
                ->whereIn('Observer Type', $info['role'])
                ->where('Is Dummy', '0')
                ->first();

            }else{
                $examProfileCountsAll = DB::table($ExamProfile)
                ->selectRaw(' count(distinct(ObserverID)) as totalBill')
                ->where('ExamType', $examType)
                ->whereIn('Observer Type', $info['role'])
                ->where('Is Dummy', '0')
                ->first();
            }

           
            $examProfileCountsUnSubmitted = DB::table($ExamProfile)
                ->selectRaw('count(distinct(ObserverID)) as totalBill')
                ->where('ExamType', $examType)
                ->whereIn('Observer Type', $info['role'])
                ->where('Is Dummy', '0')
                ->where('Submitted Documents', '0')
                ->first();

            $observerList = DB::table("$tableName as t1") // Alias for table
                ->join("$ExamProfile as t2", 't1.' . 'Observer ID', '=', 't2.ObserverID') // Fixed column format
                ->where('t1.' . $info['exam_type_column'], $examType)
                ->where('t2.' . 'Is Dummy', 0)
                ->whereIn('t2.' . 'Observer Type', $info['role'])
                ->where('t2.' . 'ExamType', $examType)
                ->selectRaw("DISTINCT (t1.[Observer ID]) as observerID")
                ->pluck('observerID')
                ->toArray();

            if(count($observerList) > 150){

                $observerChunks = array_chunk($observerList, 100); // 100 is a safe chunk size
                $totals = [
                    'open' => 0,
                    'processing' => 0,
                    'rejected' => 0,
                    'completed' => 0
                ];
    
                foreach ($observerChunks as $chunk) {
                    $partial = DB::table($tableName)
                        ->whereIn('Observer ID', $chunk)
                        ->where($info['exam_type_column'], $examType)
                        ->selectRaw("
                                 SUM(CASE WHEN [Bill Status] = ? THEN 1 ELSE 0 END) AS [open],
                                 SUM(CASE WHEN [Bill Status] IN (?, ?, ?) THEN 1 ELSE 0 END) AS [processing],
                                 SUM(CASE WHEN [Bill Status] = ? THEN 1 ELSE 0 END) AS [rejected],
                                 SUM(CASE WHEN [Bill Status] = ? THEN 1 ELSE 0 END) AS [completed]
                                 ", [0, 1, 2, 4, 3, 5])
                    
                        ->first();
    
                    // Add partial results to the total
                    foreach ($totals as $key => &$value) {
                        $value += $partial->$key ?? 0;
                    }
                }
                $billCounts = (object) $totals;
            }else{
                  $billCounts = DB::table($tableName)
                    ->whereIn('Observer ID', $observerList)
                    ->where($info['exam_type_column'], $examType)
                    ->selectRaw("
                        SUM(CASE WHEN [Bill Status] = ? THEN 1 ELSE 0 END) AS [open],
                        SUM(CASE WHEN [Bill Status] IN (?, ?, ?) THEN 1 ELSE 0 END) AS [processing],
                        SUM(CASE WHEN [Bill Status] = ? THEN 1 ELSE 0 END) AS [rejected],
                        SUM(CASE WHEN [Bill Status] = ? THEN 1 ELSE 0 END) AS [completed]
                    ", [0, 1, 2, 4, 3, 5])
                    ->first();
            }

            if ($billCounts) {
                $data[] = [
                    "type" => $info['type'],
                    "table" => $info['table'],
                    "examType" =>  $examType,
                    "status" => [
                        "open" => (int)$billCounts->open,
                        "processing" => (int)$billCounts->processing,
                        "rejected" => (int)$billCounts->rejected,
                        "completed" => (int)$billCounts->completed,
                        "unsubmitted" => (int)$examProfileCountsUnSubmitted->totalBill,
                        "totalbill" => (int)$examProfileCountsAll->totalBill
                    ],
                    "labelName" => [
                        "open" => 'Open',
                        "processing" => 'In Process Bill',
                        "rejected" => 'Rejected',
                        "completed" => 'Payment Completed',
                        "unsubmitted" => 'Unsubmitted',
                        "totalbill" => 'Total Bill'
                    ],
                    "statusKey" => [
                        "open" => 0,
                        "processing" => 1,
                        "rejected" => 3,
                        "completed" => 5,
                        "unsubmitted" => 'UN',
                        "totalbill" => 'TOT'
                    ],
                ];
            } else {
                $data[] = [
                    "type" => $info['type'],
                    "table" => $info['table'],
                    "status" => [
                        "open" => 0,
                        "processing" => 0,
                        "rejected" => 0,
                        "completed" => 0,
                        "unsubmitted" => 0,
                        "totalbill" => 0,
                    ],
                ];
            }
        }
        
        return response()->json([
            'success' => true,
            'data' => $data,
        ]);
    }

    public function getAllBilldata()
    {

        $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
        $TABillHeaderTbl = config('constants.TABillHeaderTbl');
        $CentreBillTbl = config('constants.CentreBillTbl');
        $ExamProfile = config('constants.ExamProfile');
        $ExamDates = config('constants.ExamDates');
        // Validate configuration constants
        if (empty($CityCoordinatorOfflineTbl) || empty($ExamProfile)) {
            return response()->json(['error' => 'Configuration constants are missing or invalid'], 400);
        }
        // Escape table names for SQL Server
        $CityCoordinatorOfflineTbl = '[' . str_replace(['[', ']'], '', $CityCoordinatorOfflineTbl) . ']';
        $CentreBillTbl = '[' . str_replace(['[', ']'], '', $CentreBillTbl) . ']';
        $ExamProfile = '[' . str_replace(['[', ']'], '', $ExamProfile) . ']';
        $TABillHeaderTbl = '[' . str_replace(['[', ']'], '', $TABillHeaderTbl) . ']';
        $ExamDates = '[' . str_replace(['[', ']'], '', $ExamDates) . ']';
        try {
                $query = "WITH BillHeader AS (
                SELECT DISTINCT '' as ReferenceNo, ObserverID as ObserverID, ed.[Exam Type Id] as ExamID, ep.CENTNO CenterNo, '5' BillStatus, role.Role_Name as examtable,[Exam Type] as ExamCode,'' as billsubmitdate from $ExamProfile as ep
                inner join [FRONT_ERP_MASTER_Role] as role on role.ID = ep.[Observer Type]
                inner join $ExamDates as ed on ep.ExamType =ed.[Exam Type]
                WHERE [Is Dummy] = 0 AND [Submitted Documents]=0
                Union All
                SELECT [Reference No_] ReferenceNo, [Observer ID] ObserverID, [Exam ID] ExamID, CenterNo, [Bill Status] BillStatus, 'CITY' as examtable , [Exam Type] as ExamCode,[Created On] as billsubmitdate FROM $CityCoordinatorOfflineTbl
                UNION ALL
                SELECT [Reference No_] ReferenceNo, [Observer ID] ObserverID, [Exam ID] ExamID, CenterNo, [Bill Status] BillStatus, 'CENTRE' as examtable ,[Exam Type] as ExamCode, [Created On] as billsubmitdate FROM $CentreBillTbl
                UNION ALL
                SELECT [Reference No_] ReferenceNo, [Observer ID] ObserverID, [Exam_ID] ExamID, [Centre No_] CenterNo, [Bill Status] BillStatus, 'OBSERVER' as examtable , [Exam Code] as ExamCode,[CreatedDate] as billsubmitdate FROM $TABillHeaderTbl
                ),
                RankedData AS (
                SELECT
                bh.ReferenceNo,
                bh.ObserverID,
                ex.Name AS ObserverName,
                da.[Exam Name] AS ExamID,
                bh.BillStatus,
                ex.[Observer Type],
                ex.Mobile,
                ex.Email,
                bh.examtable,
                ex.ExamType,
                ex.CENTNO,
                ex.BillType,
                bh.ExamCode,
                bh.billsubmitdate,
                ROW_NUMBER() OVER (PARTITION BY bh.ReferenceNo ORDER BY bh.ObserverID DESC) AS RowNum
                FROM BillHeader bh
                INNER JOIN $ExamProfile ex ON ex.ObserverID = bh.ObserverID and bh.ExamCode = ex.ExamType
                --AND ex.CENTNO = bh.CenterNo
                AND CHARINDEX(',' + CAST(ex.CENTNO AS VARCHAR) + ',', ',' + bh.CenterNo + ',') > 0
                INNER JOIN $ExamDates da ON da.[Exam Type Id] = bh.ExamID
                )
                SELECT
                ReferenceNo,
                ObserverID,
                ObserverName,
                ExamID,
                CASE
                WHEN BillStatus = 0 THEN 'Open'
                WHEN BillStatus = 1 THEN 'Paid'
                WHEN BillStatus = 2 THEN 'Pending'
                ELSE 'Unsubmitted'
                END AS BillStatus,
                CASE
                WHEN [Observer Type] = 1 THEN 'City Coordinator'
                WHEN [Observer Type] = 14 THEN 'Centre Superintendent'
                WHEN [Observer Type] = 2 THEN 'Observer'
                WHEN [Observer Type] = 5 THEN 'Observer'
                ELSE '-'
                END AS Role,
                [Observer Type] RoleValue,
                Mobile,
                Email,
                examtable,
                ExamType,
                CENTNO as Centno,
                BillType,
                --billsubmitdate
               CASE 
                    WHEN billsubmitdate IS NULL 
                        OR LTRIM(RTRIM(CONVERT(VARCHAR, billsubmitdate, 120))) = '' 
                        OR billsubmitdate = '1990-01-01'
                    THEN NULL
                    ELSE FORMAT(billsubmitdate, 'dd-MM-yyyy')
                END AS billsubmitdate
                FROM RankedData
                --WHERE RowNum = 1
                ORDER BY ReferenceNo DESC";

                $AllBilldata = DB::select(DB::raw($query));
                 //prd($AllBilldata);
                return response()->json($AllBilldata, 200);
            } catch (\Exception $e) {
                return response()->json([
                'error' => 'An error occurred while fetching data',
                'details' => $e->getMessage()
                ], 500);
            }
    }
    public function billDetails(Request $request)
    {
        try {
            // Fetching table names from config constants
            $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
            $TABillHeaderTbl = config('constants.TABillHeaderTbl');
            $CentreBillTbl = config('constants.CentreBillTbl');
            $VendorBankTbl = config('constants.VendorBankTbl');
            $VendorPanTbl = config('constants.VendorPanTbl');
            $VendorContactTbl = config('constants.VendorContactTbl');
            $ExamProfile = config('constants.ExamProfile');
            $TABillLocalTbl = config('constants.TABillLocalTbl');
            $TABillHotelTbl = config('constants.TABillHotelTbl');
            $TABillOutstationTbl = config('constants.TABillOutstationTbl');
            $examTable = config('constants.ExamDates');
            $CandidateCountTable=config('constants.CandidateCountTable');

            // Clean table names by removing square brackets
            $CityCoordinatorOfflineTbl = '[' . str_replace(['[', ']'], '', $CityCoordinatorOfflineTbl) . ']';
            $TABillHeaderTbl = '[' . str_replace(['[', ']'], '', $TABillHeaderTbl) . ']';
            $CentreBillTbl = '[' . str_replace(['[', ']'], '', $CentreBillTbl) . ']';
            $VendorBankTbl = '[' . str_replace(['[', ']'], '', $VendorBankTbl) . ']';
            $VendorPanTbl = '[' . str_replace(['[', ']'], '', $VendorPanTbl) . ']';
            $VendorContactTbl = '[' . str_replace(['[', ']'], '', $VendorContactTbl) . ']';
            $ExamProfile = '[' . str_replace(['[', ']'], '', $ExamProfile) . ']';
            $TABillLocalTbl = '[' . str_replace(['[', ']'], '', $TABillLocalTbl) . ']';
            $TABillHotelTbl = '[' . str_replace(['[', ']'], '', $TABillHotelTbl) . ']';
            $TABillOutstationTbl = '[' . str_replace(['[', ']'], '', $TABillOutstationTbl) . ']';
            $examTable = '[' . str_replace(['[', ']'], '', $examTable) . ']';
            $CandidateCountTable = '[' . str_replace(['[', ']'], '', $CandidateCountTable) . ']';

            // Validate incoming request data
            $validated = $request->validate([
                'referenceNo' => 'required|string',
                'observerId' => 'required|string',
                'nextval' => 'required|string',
                'examtable' => 'required|string',
                'status' => 'required|string',
                'type' => 'required|string',
                'verify' => 'required|string',
                'restriction' => 'nullable|string',
            ]);
            // print_r($validated);
            // die;
            $referenceNo = $validated['referenceNo'];
            $observerId = $validated['observerId'];
            $nextval = $validated['nextval'];
            $billTable = $validated['examtable'];
            $Billstatus = $validated['status'];
            $examType = $validated['type'];
            $verify = $validated['verify'];
            $restriction = $validated['restriction'];

            $TABillLocalData=[];
            $TABillHotelData=[];
            $TABillOutstationData=[];
            $data = null;
            // prd($validated);
            // update bill status
            $success=0;
            if($verify !='no')
            {

                if($verify=='rejected')
                {
                    if ($billTable=='CITY') {
                    
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Exam Type]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='3';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CityCoordinatorOfflineTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log  

                    $result= DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='3'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                            'table' => "$CityCoordinatorOfflineTbl",
                        ]);
                   $success = $result ? 1 : 0;
                    }
                    else if ($billTable=='CENTRE') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $CentreBillTbl where [Reference No_]='$referenceNo' and [Exam Type]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='3';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CentreBillTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log  
                       $result= DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='3'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                            'table' => "$CentreBillTbl",
                        ]);
                    $success = $result ? 1 : 0;
                    }
                    else if ($billTable=='OBSERVER') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Exam Code]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='3';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$TABillHeaderTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log  
                        $result=  DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='3'",
                            'where' => "where  [Exam Code]='$examType' and [Reference No_] ='$referenceNo' ",
                            'table' => "$TABillHeaderTbl",
                        ]);
                        $success = $result ? 1 : 0;
                    //log start
                        $getDetail['Submitted Documents']=1;
                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Submitted Documents']='0';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$TABillHeaderTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log  
                        $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Submitted Documents]='0'",
                            'where' => "where  [ExamType]='$examType' and [ObserverID] ='$observerId'",
                            'table' => "$ExamProfile",
                        ]);
                        $success = $result ? 1 : 0;
                    }
                }else if($verify=='approved')
                {

                    if ($billTable=='CITY') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Exam Type]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='4';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CityCoordinatorOfflineTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log                          
                        $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='4'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                            'table' => "$CityCoordinatorOfflineTbl",
                        ]);
                        $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='CENTRE') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $CentreBillTbl where [Reference No_]='$referenceNo' and [Exam Type]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='4';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CentreBillTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log  
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='4'",
                                'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$CentreBillTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='OBSERVER') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Exam Code]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='4';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$TABillHeaderTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log                              
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='4'",
                                'where' => "where  [Exam Code]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$TABillHeaderTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                }else if($verify==='verifiedYes')
                {

                    if ($billTable=='CITY') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Exam Type]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='1';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CityCoordinatorOfflineTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log   
                        $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='1'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                            'table' => "$CityCoordinatorOfflineTbl",
                        ]);
                        $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='CENTRE') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $CentreBillTbl where [Reference No_]='$referenceNo' and [Exam Type]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='1';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CentreBillTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log 
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='1'",
                                'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$CentreBillTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='OBSERVER') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Exam Code]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='1';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$TABillHeaderTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log  
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='1'",
                                'where' => "where  [Exam Code]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$TABillHeaderTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                }else if($verify=='complete')
                {
                    if ($billTable=='CITY') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Exam Type]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='5';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CityCoordinatorOfflineTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log                           
                        $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='5'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                            'table' => "$CityCoordinatorOfflineTbl",
                        ]);
                        $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='CENTRE') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $CentreBillTbl where [Reference No_]='$referenceNo' and [Exam Type]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='5';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CentreBillTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log 
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='5'",
                                'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$CentreBillTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='OBSERVER') {
                    //log start
                        //select old value                          
                        $query = "select [Bill Status]  from
                        $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Exam Code]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value['Bill Status']='5';                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$TABillHeaderTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log  
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='5'",
                                'where' => "where  [Exam Code]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$TABillHeaderTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                }
            }
            // end update

            if($nextval=='yes' && $restriction=='no')
            {

                if ($Billstatus == 1) {
                    $status = "AND [Bill Status] IN ('1','2','4') AND  [Reference No_] !='$referenceNo' ORDER BY NEWID() ";
                }else{
                    $status = "AND [Bill Status]='$Billstatus' AND  [Reference No_] !='$referenceNo'  ORDER BY NEWID()";

                }
                if ($billTable=='CITY') {
                $MaxCountData=[];
                $query = "select TOP 1 * from $CityCoordinatorOfflineTbl where [Exam Type]='$examType' $status";
                $data = DB::select(DB::raw($query));
                $viewLoad = 'ccfields';

                }
                else if ($billTable=='CENTRE') {
                    $query = "select TOP 1 * from $CentreBillTbl where [Exam Type]='$examType' $status";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'csfields';
                    
                    $query2 = "select TOP 1  D1Max,D2Max,D3Max,D4Max,D5Max,D6Max,D7Max,D8Max,D9Max,D10Max,D11Max,D12Max,[Total Candidate Count in Exam] from $CandidateCountTable where [Exam Type]='$examType'";
                    $MaxCountData = DB::select(DB::raw($query2));
                    
                }

                else if ($billTable=='OBSERVER') {
                    $MaxCountData=[];
                    $query = "select TOP 1 * from $TABillHeaderTbl where [Exam Code]='$examType' $status";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'observerfields';

                    $observerId=$data[0]->{'Observer ID'};
                    $referenceNo=$data[0]->{'Reference No_'};

                    $query = "select * from $TABillLocalTbl where [Reference No_]='$referenceNo'";
                    $TABillLocalData = DB::select(DB::raw($query));
                    $query = "select * from $TABillHotelTbl where [Reference No_]='$referenceNo'";
                    $TABillHotelData = DB::select(DB::raw($query));
                    $query = "select * from $TABillOutstationTbl where [Reference No_]='$referenceNo'";
                    $TABillOutstationData = DB::select(DB::raw($query));
                }

                    $observerId=$data[0]->{'Observer ID'};
                    $referenceNo=$data[0]->{'Reference No_'};

            }else {
                $MaxCountData=[];
                $query = "select * from $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                $data = DB::select(DB::raw($query));
                $viewLoad = 'ccfields';


                if (empty($data)) {
                    $query = "select * from $CentreBillTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'csfields';
                    $query2 = "select TOP 1 D1Max,D2Max,D3Max,D4Max,D5Max,D6Max,D7Max,D8Max,D9Max,D10Max,D11Max,D12Max,[Total Candidate Count in Exam] from $CandidateCountTable where [Exam Type]='$examType'";
                    $MaxCountData = DB::select(DB::raw($query2));
                }

                if (empty($data)) {
                    $MaxCountData=[];
                    $query = "select * from $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'observerfields';

                    $query = "select * from $TABillLocalTbl where [Reference No_]='$referenceNo'";
                    $TABillLocalData = DB::select(DB::raw($query));
                    $query = "select * from $TABillHotelTbl where [Reference No_]='$referenceNo'";
                    $TABillHotelData = DB::select(DB::raw($query));
                    $query = "select * from $TABillOutstationTbl where [Reference No_]='$referenceNo'";
                    $TABillOutstationData = DB::select(DB::raw($query));
                }
            }
            
            // If no data was found from any of the tables
            if (empty($data)) {
                return response()->json(['error' => 'No records found for the provided reference number and observer ID.'], 404);
            }

            $emode = "select Exam_Mode from $examTable where [Exam Type]='$examType'";
            $exam_mode = DB::select(DB::raw($emode));

            $examMode=$exam_mode[0]->{'Exam_Mode'};

            if($examMode)
            {
                if($examMode=='CBT')
                {
                   $Mode='Online';
                }elseif($examMode=='PNP')
                {
                    $Mode='Offline';
                }
            }
            $query = "select * from $VendorBankTbl where [No_]='$observerId'";
            $VendorBank = DB::select(DB::raw($query));
            $query = "select [P_A_N_ No_] from $VendorPanTbl where [No_]='$observerId'";
            $VendorPan = DB::select(DB::raw($query));
            $query = "select * from $VendorContactTbl where [No_]='$observerId'";
            $VendorContact = DB::select(DB::raw($query));
            $query = "select [Observer Type] from $ExamProfile where [ObserverID]='$observerId' and [ExamType]='$examType'";
            $ExamProfile = DB::select(DB::raw($query));

            $VendorBank=$VendorBank[0]??[];
            $VendorPan=$VendorPan[0]??[];
            $VendorContact=$VendorContact[0]??[];
            $tCount=count($data);
            $data=$data[0]??[];
            $role=config('constants.RoleName.'.$ExamProfile[0]->{'Observer Type'});
            $referenceNo=$data->{'Reference No_'};
            $BillStatus=$data->{'Bill Status'};
            $Role_ID=auth()->user()->Role_ID;
            $MaxCountVal=$MaxCountData[0]??[];
            // prd($MaxCountData[0]->D1Max);
            $htmlView = view('admin.'.$viewLoad, compact('data', 'VendorBank', 'VendorPan', 'VendorContact', 'role', 'examType', 'Role_ID','billTable','Billstatus','TABillLocalData', 'TABillHotelData', 'TABillOutstationData', 'Mode', 'restriction','referenceNo','tCount','MaxCountVal'))->render();

            // Return JSON with both the view and additional data
            return response()->json([
                'html' => $htmlView,  // This contains the full view
                'role' => $role,
                'Mode' => $Mode,
                'Role_ID' => $Role_ID,
                'restriction' => $restriction,
                'BillStatus' => $BillStatus,
                'updateStatus'=>$success,
                'referenceNo' => $referenceNo,
                'tCount'=>$tCount,
                'MaxCountVal'=>$MaxCountVal
            ]);

        } catch (\Exception $e) {
            // Handle any errors
            return response()->json(['error' => 'Query execution failed: ' . $e->getMessage()], 500);
        }
    }

    public function getListingByLabel(Request $request)
    {
        // Validate the input parameters
        $validated = $request->validate([
            'label' => 'required|string',
            'type' => 'required|string',
            'status' => 'required|string',
            'examType' => 'required|string',
        ]);

        // Extract values from validated request
        $label = $validated['label'];
        $type = $validated['type'];
        $examType = $validated['examType'];

        // Set status and condition based on the input status
        $status = '';
        $THERCONDITION = '';
        if ($validated['status'] == 1) {
            $status = "AND CCTbl.[Bill Status] IN ('1','2','4')";
        } elseif ($validated['status'] == 'UN') {
            $THERCONDITION = "AND EX.[Submitted Documents] = '0'";
            $status = '';
        } elseif ($validated['status'] == 'TOT') {
            $status = '';
            $THERCONDITION = '';
        } else {
            $status = 'AND CCTbl.[Bill Status] IN (' . $validated['status'] . ')';
        }


        // Get table names from config constants
        $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
        $TABillHeaderTbl = config('constants.TABillHeaderTbl');
        $CentreBillTbl = config('constants.CentreBillTbl');
        $ExamProfile = config('constants.ExamProfile');

        // Sanitize table names to avoid issues with square brackets
        $sanitizeTableName = function ($table) {
            return '[' . str_replace(['[', ']'], '', $table) . ']';
        };

        // Clean the table names
        $CityCoordinatorOfflineTbl = $sanitizeTableName($CityCoordinatorOfflineTbl);
        $TABillHeaderTbl = $sanitizeTableName($TABillHeaderTbl);
        $CentreBillTbl = $sanitizeTableName($CentreBillTbl);
        $ExamProfile = $sanitizeTableName($ExamProfile);

        // Initialize default query components
        $SELECT = '';
        $WHERE = '';
        $TABLE = '';
        $queryType = 0;
        // Build query based on type
        switch ($type) {
            case 'CITY COORDINATOR BILL':
                if ($label == 'Unsubmitted') {
                    $queryType = 1;
                    $SELECT = "Select DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                             STRING_AGG(CENTNO, ',') AS Centno,
                            [Observer Type] as [role],
                            Email,
                            '0000-00-00 00:00:00.000'  as billsubmitdate from $ExamProfile
                            WHERE [ExamType] = '$examType' AND [Is Dummy] = 0 AND [Submitted Documents]=0  AND  [Observer Type]  in(1)
                            GROUP BY 
                            [ObserverID],
                            Name,
                            [ExamType],
                            Mobile, 
                            BillType,
                            [Observer Type],
                            Email
                            ";
                    // $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0 AND [Submitted Documents]=0  AND  [Observer Type]  in(1) ";
                    // $TABLE = "$ExamProfile";
                } else if ($label == 'Total Bill') {
                    $queryType = 1;
                    $SELECT = "Select DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            STRING_AGG(CENTNO, ',') AS Centno,
                            [Observer Type] as [role],
                            Email,
                            '0000-00-00 00:00:00.000'  as billsubmitdate from $ExamProfile
                            WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(1)
                            GROUP BY 
                            [ObserverID],
                            Name,
                            [ExamType],
                            Mobile, 
                            BillType,
                            [Observer Type],
                            Email
                            ";
                    
                } else {
                    $queryType = 1;
                    $SELECT = "select CCTbl.[Reference No_] AS ReferenceNo,
                            CCTbl.IsNewBill AS BillType,
                            EX.[ObserverID] AS ObserverID,
                            EX.[Name] AS ObserverName,
                            EX.[ExamType] AS ExamID,
                            EX.Mobile,
                            STRING_AGG(EX.CENTNO, ',') AS Centno,
                            EX.[Observer Type] AS [role],
                            EX.Email,
                            [Created On] as billsubmitdate from $ExamProfile AS EX
                            INNER JOIN $CityCoordinatorOfflineTbl as CCTbl ON EX.[ObserverID] = CCTbl.[Observer ID] AND EX.[ExamType] = CCTbl.[Exam Type] 
                            WHERE EX.[ExamType] = '$examType' $status AND EX.[Is Dummy] = 0 $THERCONDITION GROUP BY 
                            CCTbl.[Reference No_],
                            CCTbl.IsNewBill,
                            EX.[ObserverID],
                            EX.[Name],
                            EX.[ExamType],
                            EX.Mobile, 
                            EX.[Observer Type],
                            EX.Email,
                            [Created On] order by CCTbl.[Reference No_]";
                   
                }
                break;

            case 'OBSERVER BILL':
                if ($label == 'Unsubmitted') {
                    $queryType = 1;
                    $SELECT = "Select DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            STRING_AGG(CENTNO, ',') AS Centno,
                            [Observer Type] as [role],
                            Email,
                            '0000-00-00 00:00:00.000'  as billsubmitdate from $ExamProfile
                            WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(5, 2, 3, 10)
                            GROUP BY 
                            [ObserverID],
                            Name,
                            [ExamType],
                            Mobile, 
                            BillType,
                            [Observer Type],
                            Email
                            ";
                    // $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0 AND [Submitted Documents]=0  AND  [Observer Type]  in(5, 2, 3, 10)";
                    // $TABLE = "$ExamProfile";
                } else if ($label == 'Total Bill') {
                    $queryType = 1;
                    $SELECT = "Select DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            STRING_AGG(CENTNO, ',') AS Centno,
                            [Observer Type] as [role],
                            Email,
                            '0000-00-00 00:00:00.000'  as billsubmitdate from $ExamProfile
                            WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(5, 2, 3, 10)
                            GROUP BY 
                            [ObserverID],
                            Name,
                            [ExamType],
                            Mobile, 
                            BillType,
                            [Observer Type],
                            Email
                            ";
                    // $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(5, 2, 3, 10)";
                    // $TABLE = "$ExamProfile";
                } else {
                    $queryType = 1;
                    $SELECT = "Select DISTINCT
                            CCTbl.[Reference No_] as ReferenceNo,
                            CCTbl.IsNewBill as BillType,
                            EX.[ObserverID] as ObserverID,
                            EX.[Name] as ObserverName,
                            EX.[ExamType] as ExamID,
                            EX.Mobile,
                            STRING_AGG(EX.CENTNO, ',') AS Centno,
                            EX.[Observer Type] as [role], 
                            EX.Email,
                            [CreatedDate] as billsubmitdate from $ExamProfile AS EX
                            INNER JOIN $TABillHeaderTbl as CCTbl ON EX.[ObserverID] = CCTbl.[Observer ID] AND EX.[ExamType] = CCTbl.[Exam Code] AND EX.[CENTNO] IN (SELECT value FROM STRING_SPLIT(CCTbl.[Centre No_], ','))
                            WHERE EX.[ExamType] = '$examType' $status AND EX.[Is Dummy] = 0 $THERCONDITION GROUP BY 
                            CCTbl.[Reference No_],
                            CCTbl.IsNewBill,
                            EX.[ObserverID],
                            EX.[Name],
                            EX.[ExamType],
                            EX.Mobile, 
                            EX.[Observer Type],
                            EX.Email,
                            [CreatedDate]
                            order by [Reference No_]";
                   
                }
                break;

            case 'CENTRE SUPERINTENDENT BILL':
                if ($label == 'Unsubmitted') {
                    $SELECT = " DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            CENTNO as Centno,
                            [Observer Type] as [role],
                            Email,
                            '0000-00-00 00:00:00.000'  as billsubmitdate";
                    $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0 AND [Submitted Documents]=0  AND  [Observer Type]  in(14)";
                    $TABLE = "$ExamProfile";
                } else if ($label == 'Total Bill') {
                    $SELECT = " DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            CENTNO as Centno,
                            [Observer Type] as [role],
                            Email,
                            '0000-00-00 00:00:00.000'  as billsubmitdate";
                    $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(14)";
                    $TABLE = "$ExamProfile";
                } else {
                    $SELECT = 'DISTINCT
                        CCTbl.[Reference No_] as ReferenceNo,
                        CCTbl.IsNewBill as BillType,
                        EX.[ObserverID] as ObserverID,
                        EX.[Name] as ObserverName,
                        EX.[ExamType] as ExamID,
                        EX.Mobile,
                        EX.CENTNO as Centno,
                        EX.[Observer Type] as [role],
                        EX.Email,
                        [Created On] as billsubmitdate';
                    $WHERE = "INNER JOIN $CentreBillTbl as CCTbl ON EX.[ObserverID] = CCTbl.[Observer ID] AND EX.[ExamType] = CCTbl.[Exam Type] AND EX.[CENTNO] IN (SELECT value FROM STRING_SPLIT(CCTbl.[CenterNo], ','))
                       WHERE EX.[ExamType] = '$examType' $status AND EX.[Is Dummy] = 0 $THERCONDITION order by CCTbl.[Reference No_]";
                    $TABLE = "$ExamProfile AS EX";
                }
                break;

            default:
                return response()->json(['error' => 'Invalid bill type'], 400);
        }

        // Execute the query using parameterized binding for stored procedure
        // Use the 'DB::select' method for executing the query.
        try {
            //  echo $SELECT,"<br>";
            // echo $WHERE,"<br>";
            // echo $TABLE;
            //  die;
            if($queryType == 1){
                $data = DB::select(DB::raw($SELECT));
            }else{
                $data = DB::select(DB::raw("EXEC getCommonDataSP :select, :input, :table"), [
                                'select' => $SELECT,
                                'input' => $WHERE,
                                'table' => $TABLE,
                                // 'orderBy' => 'ORDER BY [Reference No_] DESC',
                            ]);
            }
           

            
        } catch (\Exception $e) {
            return response()->json(['error' => 'Query execution failed: ' . $e->getMessage()], 500);
        }

        // Return the result as a JSON response
        return response()->json($data);
    }

    public function updateBillDeatils(Request $request)
    {

        try {
            // Fetching table names from config constants
            $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
            $TABillHeaderTbl = config('constants.TABillHeaderTbl');
            $CentreBillTbl = config('constants.CentreBillTbl');
            $VendorBankTbl = config('constants.VendorBankTbl');
            $VendorPanTbl = config('constants.VendorPanTbl');
            $VendorContactTbl = config('constants.VendorContactTbl');
            $ExamProfile = config('constants.ExamProfile');
            $TABillLocalTbl = config('constants.TABillLocalTbl');
            $TABillHotelTbl = config('constants.TABillHotelTbl');
            $TABillOutstationTbl = config('constants.TABillOutstationTbl');
            $examTable = config('constants.ExamDates');
            $CandidateCountTable=config('constants.CandidateCountTable');
            // Clean table names by removing square brackets
            $CityCoordinatorOfflineTbl = '[' . str_replace(['[', ']'], '', $CityCoordinatorOfflineTbl) . ']';
            $TABillHeaderTbl = '[' . str_replace(['[', ']'], '', $TABillHeaderTbl) . ']';
            $CentreBillTbl = '[' . str_replace(['[', ']'], '', $CentreBillTbl) . ']';
            $VendorBankTbl = '[' . str_replace(['[', ']'], '', $VendorBankTbl) . ']';
            $VendorPanTbl = '[' . str_replace(['[', ']'], '', $VendorPanTbl) . ']';
            $VendorContactTbl = '[' . str_replace(['[', ']'], '', $VendorContactTbl) . ']';
            $ExamProfile = '[' . str_replace(['[', ']'], '', $ExamProfile) . ']';
            $TABillLocalTbl = '[' . str_replace(['[', ']'], '', $TABillLocalTbl) . ']';
            $TABillHotelTbl = '[' . str_replace(['[', ']'], '', $TABillHotelTbl) . ']';
            $TABillOutstationTbl = '[' . str_replace(['[', ']'], '', $TABillOutstationTbl) . ']';
            $examTable = '[' . str_replace(['[', ']'], '', $examTable) . ']';
            $CandidateCountTable = '[' . str_replace(['[', ']'], '', $CandidateCountTable) . ']';

            // Validate incoming request data
            $validated = $request->validate([
                'referenceNo' => 'required|string',
                'observerId' => 'required|string',
                'examtable' => 'required|string',
                'status' => 'required|string',
                'type' => 'required|string',
                'updateId' => 'required|string',
                'getupdate' => 'nullable|string',
                'restriction' => 'nullable|string',

            ]);
            $referenceNo = $validated['referenceNo'];
            $observerId = $validated['observerId'];
            $billTable = $validated['examtable'];
            $Billstatus = $validated['status'];
            $examType = $validated['type'];
            $updateId = $validated['updateId'];
            $updateValue = $validated['getupdate'];
            $restriction = $validated['restriction'];
            // print_R($validated);
            // die;
            $TABillLocalData=[];
            $TABillHotelData=[];
            $TABillOutstationData=[];
            $data = null;

            // update bill status
            $columnName=str_replace('###',' ',$updateId);

            $success='Your changes have been updated successfully.';

                if ($billTable=='CITY') {
                   if(in_array($columnName,['Remark','Rejection Remark','No_ of Candidate Alloted NTA'])){
                    
                    //log start
                        //select old value  
                        
                        $query = "select [$columnName]  from
                        $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Exam Type]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value[$columnName]=$updateValue;                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CityCoordinatorOfflineTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log

                    DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                        'updatefields' => "[$columnName]='$updateValue'",
                        'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                        'table' => "$CityCoordinatorOfflineTbl",
                    ]);

                    }else if(in_array($columnName,['Advance Payment','TDS_Amount'])){
                        
                        $query = "select [$columnName],[Net Amount Payable],[Total Deduction Amount],[Total Expenditure Amount],[Observer TDS],[Deputy Observer TDS] from
                        $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                        $getDetail = DB::select(DB::raw($query));

                        $deduction=$getDetail[0]->{$columnName};
                        $Expenditure=$getDetail[0]->{'Total Expenditure Amount'};
                        $Observer=$getDetail[0]->{'Observer TDS'};
                        $Deputy=$getDetail[0]->{'Deputy Observer TDS'};
                        $TotalDeductionAmount=$getDetail[0]->{'Total Deduction Amount'};
                        $NetAmountPayable=$Expenditure-($Observer+$Deputy+$deduction);
                        $NetPayableAfterDeduction=($NetAmountPayable  - $TotalDeductionAmount);
                      
                        
                    //log start 
                        $logData['current_value']=json_encode($getDetail);
                        $Update_value[$columnName]=$updateValue;
                        $Update_value['Net Amount Payable']=$NetAmountPayable;
                        $Update_value['Net Payable After Deduction]']=$NetPayableAfterDeduction;                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CityCoordinatorOfflineTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId;
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log

                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[$columnName]='$updateValue',[Net Amount Payable]='$NetAmountPayable',[Net Payable After Deduction]='$NetPayableAfterDeduction'",
                                'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                                'table' => "$CityCoordinatorOfflineTbl",
                            ]);

                    }
                    else{
                        
                        $query = "select [$columnName],[Total Deduction Amount],[Net Amount Payable] from
                        $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                        $getDetail = DB::select(DB::raw($query));
                        $deduction=$getDetail[0]->{$columnName};
                        $TotalDeductionAmount=$getDetail[0]->{'Total Deduction Amount'};
                        $TotalDeductionAmount=($TotalDeductionAmount-$deduction) + $updateValue;
                        $NetPayableAmount=$getDetail[0]->{'Net Amount Payable'};
                        $NetPayableAfterDeduction=$NetPayableAmount - ($TotalDeductionAmount);

                        //log start 
                            $logData['current_value']=json_encode($getDetail);
                            $Update_value[$columnName]=$updateValue;
                            $Update_value['Total Deduction Amount']=$TotalDeductionAmount;
                            $Update_value['Net Payable After Deduction]']=$NetPayableAfterDeduction;                            
                            $logData['update_value']=json_encode($Update_value);
                            $logData['table_name']=$CityCoordinatorOfflineTbl;
                            $logData['reference_no']=$referenceNo;
                            $logData['exam_type']=$examType;
                            $logData['observer_id']=$observerId;
                            $logData['created_at']=date('Y-m-d h:i:s');
                            MasterServices::tbl_changeslog($logData);
                        //end log

                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[$columnName]='$updateValue',[Total Deduction Amount]='$TotalDeductionAmount',[Net Payable After Deduction]='$NetPayableAfterDeduction'",
                                'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                                'table' => "$CityCoordinatorOfflineTbl",
                            ]);
                    }

                    }
                    else if ($billTable=='CENTRE') {
                        if(in_array($columnName,['Remarks','Rejection Remark','Day1 Count','Total Candidate Count Exam'])){
                    //log start
                        //select old value  
                        
                        $query = "select [$columnName]  from
                        $CentreBillTbl where [Reference No_]='$referenceNo' and [Exam Type]='$examType'";
                        $getDetail = DB::select(DB::raw($query));   
                        //end

                        $logData['current_value']=json_encode($getDetail);
                        $Update_value[$columnName]=$updateValue;                        
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$CentreBillTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId??'-';
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log                       
                       
                       
                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                        'updatefields' => "[$columnName]='$updateValue'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                            'table' => "$CentreBillTbl",
                        ]);
                        }else if(in_array($columnName,['Amount Rec_ in Advance'])){
                            
                            $query = "select [$columnName],[Net Amount Payable],[Total Deductions Amount],[Total Expenditure Amount],[TDS_Amount] from
                            $CentreBillTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                            $getDetail = DB::select(DB::raw($query));
                            $deduction=$getDetail[0]->{$columnName};
                            $NetAmountPayable=$getDetail[0]->{'Net Amount Payable'};
                            $TDS_Amount=$getDetail[0]->{'TDS_Amount'};
                            $TotalExpenditureAmount=$getDetail[0]->{'Total Expenditure Amount'};
                            $TotalDeductionsAmount=$getDetail[0]->{'Total Deductions Amount'};
                            $NetAmountPayable=($TotalExpenditureAmount - ($updateValue + $TDS_Amount));
                            $NetPayableAfterDeduction=$NetAmountPayable - $TotalDeductionsAmount;
                            
                            //log start 
                            $logData['current_value']=json_encode($getDetail);
                            $Update_value[$columnName]=$updateValue;
                            $Update_value['Net Amount Payable']=$NetAmountPayable;
                            $Update_value['Net Payable After Deductions]']=$NetPayableAfterDeduction;                            
                            $logData['update_value']=json_encode($Update_value);
                            $logData['table_name']=$CentreBillTbl;
                            $logData['reference_no']=$referenceNo;
                            $logData['exam_type']=$examType;
                            $logData['observer_id']=$observerId;
                            $logData['created_at']=date('Y-m-d h:i:s');
                            MasterServices::tbl_changeslog($logData);
                            //end log

                            DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[$columnName]='$updateValue',[Net Amount Payable]='$NetAmountPayable',[Net Payable After Deductions]='$NetPayableAfterDeduction'",
                                    'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                                    'table' => "$CentreBillTbl",
                                ]);

                        }
                        else{
                            
                            $query = "select [$columnName],[Total Deductions Amount],[Net Amount Payable] from
                            $CentreBillTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                            $getDetail = DB::select(DB::raw($query));
                            $deduction=$getDetail[0]->{$columnName};
                            $TotalDeductionAmount=$getDetail[0]->{'Total Deductions Amount'};
                            $TotalDeductionAmount=($TotalDeductionAmount-$deduction) + $updateValue;
                            $NetPayableAmount=$getDetail[0]->{'Net Amount Payable'};
                            $NetPayableAfterDeduction=$NetPayableAmount - ($TotalDeductionAmount);
                            
                            //log start 
                            $logData['current_value']=json_encode($getDetail);
                            $Update_value[$columnName]=$updateValue;
                            $Update_value['Total Deductions Amount']=$TotalDeductionAmount;
                            $Update_value['Net Payable After Deductions]']=$NetPayableAfterDeduction;                            
                            $logData['update_value']=json_encode($Update_value);
                            $logData['table_name']=$CentreBillTbl;
                            $logData['reference_no']=$referenceNo;
                            $logData['exam_type']=$examType;
                            $logData['observer_id']=$observerId;
                            $logData['created_at']=date('Y-m-d h:i:s');
                            MasterServices::tbl_changeslog($logData);
                            //end log
                            DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[$columnName]='$updateValue',[Total Deductions Amount]='$TotalDeductionAmount',[Net Payable After Deductions]='$NetPayableAfterDeduction'",
                                    'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                                    'table' => "$CentreBillTbl",
                                ]);
                        }

                    }
                    else if ($billTable=='OBSERVER') {
                        if(in_array($columnName,['Remark','Rejection Remark','CreatedDate','Completed Date'])){
                        //log start
                            //select old value  
                            
                            $query = "select [$columnName]  from
                            $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Exam Code]='$examType'";
                            $getDetail = DB::select(DB::raw($query));   
                            //end

                            $logData['current_value']=json_encode($getDetail);
                            $Update_value[$columnName]=$updateValue;                            
                            $logData['update_value']=json_encode($Update_value);
                            $logData['table_name']=$TABillHeaderTbl;
                            $logData['reference_no']=$referenceNo;
                            $logData['exam_type']=$examType;
                            $logData['observer_id']=$observerId??'-';
                            $logData['created_at']=date('Y-m-d h:i:s');
                            MasterServices::tbl_changeslog($logData);
                        //end log                          
                        
                        
                        
                        
                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                        'updatefields' => "[$columnName]='$updateValue'",
                            'where' => "where  [Exam Code]='$examType' and [Reference No_] ='$referenceNo' ",
                            'table' => "$TABillHeaderTbl",
                        ]);
                    }else if( $columnName=='P_A_N_ No_')
                        {
                        //log start
                            //select old value  
                            
                            $query = "select [$columnName]  from
                            $VendorPanTbl where [No_]='$observerId'";
                            $getDetail = DB::select(DB::raw($query));   
                            //end

                            $logData['current_value']=json_encode($getDetail);
                            $Update_value[$columnName]=$updateValue;                            
                            $logData['update_value']=json_encode($Update_value);
                            $logData['table_name']=$VendorPanTbl;
                            $logData['reference_no']=$referenceNo??'-';
                            $logData['exam_type']=$examType??'-';
                            $logData['observer_id']=$observerId;
                            $logData['created_at']=date('Y-m-d h:i:s');
                            MasterServices::tbl_changeslog($logData);
                        //end log           

                            DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[$columnName]='$updateValue'",
                                    'where' => "where  [No_] ='$observerId' ",
                                    'table' => "$VendorPanTbl",
                                ]);
                    }else if(in_array($columnName,['Advance Payment']))
                    {
                        $query = "select [Net Payable Amount],[Claimed Amount],[TDS Amount],[Total Deduction Amount] from
                        $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                        $getDetail = DB::select(DB::raw($query));
                        $ClaimedAmount=$getDetail[0]->{'Claimed Amount'};
                        $TDSAmount=$getDetail[0]->{'TDS Amount'};
                        $NetPayableAmount=$getDetail[0]->{'Net Payable Amount'};
                        $NetPayableAmount=$ClaimedAmount -($TDSAmount+$updateValue);
                        $TotalDeductionAmount=$getDetail[0]->{'Total Deduction Amount'};
                        $NetPayableAfterDeduction=$NetPayableAmount - ($TotalDeductionAmount);
                     //log start   
                        $logData['current_value']=json_encode($getDetail);
                        $Update_value[$columnName]=$updateValue;                            
                        $Update_value['Net Payable Amount']=$NetPayableAmount;                            
                        $Update_value['Net Payable After Deduction']=$NetPayableAfterDeduction;                       
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$TABillHeaderTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId;
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                    //end log   
                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[$columnName]='$updateValue',[Net Payable Amount]='$NetPayableAmount',[Net Payable After Deduction]='$NetPayableAfterDeduction'",
                                'where' => "where  [Exam Code]='$examType' and [Reference No_] ='$referenceNo' ",
                                'table' => "$TABillHeaderTbl",
                            ]);
                    }else
                    {
                        $query = "select [$columnName],[Total Deduction Amount],[Net Payable Amount] from
                        $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                        $getDetail = DB::select(DB::raw($query));
                        $deduction=$getDetail[0]->{$columnName};
                        $TotalDeductionAmount=$getDetail[0]->{'Total Deduction Amount'};
                        $TotalDeductionAmount=($TotalDeductionAmount-$deduction) + $updateValue;
                        $NetPayableAmount=$getDetail[0]->{'Net Payable Amount'};
                        $NetPayableAfterDeduction=$NetPayableAmount - ($TotalDeductionAmount);
                       
                        //log start   
                        $logData['current_value']=json_encode($getDetail);
                        $Update_value[$columnName]=$updateValue;                            
                        $Update_value['Total Deduction Amount']=$TotalDeductionAmount;                            
                        $Update_value['Net Payable After Deduction']=$NetPayableAfterDeduction;                       
                        $logData['update_value']=json_encode($Update_value);
                        $logData['table_name']=$TABillHeaderTbl;
                        $logData['reference_no']=$referenceNo;
                        $logData['exam_type']=$examType;
                        $logData['observer_id']=$observerId;
                        $logData['created_at']=date('Y-m-d h:i:s');
                        MasterServices::tbl_changeslog($logData);
                        //end log   

                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[$columnName]='$updateValue',[Total Deduction Amount]='$TotalDeductionAmount',[Net Payable After Deduction]='$NetPayableAfterDeduction'",
                                'where' => "where  [Exam Code]='$examType' and [Reference No_] ='$referenceNo' ",
                                'table' => "$TABillHeaderTbl",
                            ]);
                    }


                    }



            // end update



                $query = "select * from $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                $data = DB::select(DB::raw($query));
                $viewLoad = 'ccfields';
                $MaxCountData=[];

                if (empty($data)) {
                    $query = "select * from $CentreBillTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'csfields';
                    $query2 = "select TOP 1 D1Max,D2Max,D3Max,D4Max,D5Max,D6Max,D7Max,D8Max,D9Max,D10Max,D11Max,D12Max,[Total Candidate Count in Exam] from $CandidateCountTable where [Exam Type]='$examType'";
                    $MaxCountData = DB::select(DB::raw($query2));
                }

                if (empty($data)) {
                    $MaxCountData=[];
                    $query = "select * from $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'observerfields';

                    $query = "select * from $TABillLocalTbl where [Reference No_]='$referenceNo'";
                    $TABillLocalData = DB::select(DB::raw($query));
                    $query = "select * from $TABillHotelTbl where [Reference No_]='$referenceNo'";
                    $TABillHotelData = DB::select(DB::raw($query));
                    $query = "select * from $TABillOutstationTbl where [Reference No_]='$referenceNo'";
                    $TABillOutstationData = DB::select(DB::raw($query));
                }


            // If no data was found from any of the tables
            if (empty($data)) {
                return response()->json(['error' => 'No records found for the provided reference number and observer ID.'], 404);
            }

            $emode = "select Exam_Mode from $examTable where [Exam Type]='$examType'";
            $exam_mode = DB::select(DB::raw($emode));

            $examMode=$exam_mode[0]->{'Exam_Mode'};

            if($examMode)
            {
                if($examMode=='CBT')
                {
                   $Mode='Online';
                }elseif($examMode=='PNP')
                {
                    $Mode='Offline';
                }
            }
            $query = "select * from $VendorBankTbl where [No_]='$observerId'";
            $VendorBank = DB::select(DB::raw($query));
            $query = "select [P_A_N_ No_] from $VendorPanTbl where [No_]='$observerId'";
            $VendorPan = DB::select(DB::raw($query));
            $query = "select * from $VendorContactTbl where [No_]='$observerId'";
            $VendorContact = DB::select(DB::raw($query));
            $query = "select [Observer Type] from $ExamProfile where [ObserverID]='$observerId' and [ExamType]='$examType'";
            $ExamProfile = DB::select(DB::raw($query));

            $VendorBank=$VendorBank[0]??[];
            $VendorPan=$VendorPan[0]??[];
            $VendorContact=$VendorContact[0]??[];
            $tCount=count($data);
            $data=$data[0]??[];
            $role=config('constants.RoleName.'.$ExamProfile[0]->{'Observer Type'});
            $referenceNo=$data->{'Reference No_'};
            $MaxCountVal=$MaxCountData[0]??[];
            // echo "1234";
            // die;
            $htmlView = view('admin.'.$viewLoad, compact('data', 'VendorBank', 'VendorPan', 'VendorContact', 'role', 'examType', 'billTable','restriction','Billstatus','TABillLocalData', 'TABillHotelData', 'TABillOutstationData', 'Mode', 'referenceNo','tCount','MaxCountVal'))->render();

            // Return JSON with both the view and additional data
            return response()->json([
                'html' => $htmlView,  // This contains the full view
                'role' => $role,
                'Mode' => $Mode,
                'restriction' => $restriction,
                'updateStatus'=>$success,
                'referenceNo' => $referenceNo,
                'tCount'=>$tCount,
                'MaxCountVal'=>$MaxCountVal,
            ]);

        } catch (\Exception $e) {
            // Handle any errors
            return response()->json(['error' => 'Query execution failed: ' . $e->getMessage()], 500);
        }
    }

    public function generatePdf($referenceNo = null, $examType = null , $observerID = null, $isNew = 1, $Role=null)
    {
        $anotherController = app()->make(OldReportController::class);
        if($isNew == 0){
            if($Role == 14){
                return $anotherController->csOldReport($referenceNo, $examType, $observerID, 'pdf');
            }elseif($Role == 1){
                return $anotherController->ccOldReport($referenceNo, $examType, $observerID, 'pdf');
            }
        }
    }


    public function generateBulkPdf(Request $request){
        $anotherController = app()->make(OldReportController::class);
        $anotherNewController = app()->make(PdfDownloadController::class);

       
        foreach($_POST['selectedBills'] as $key => $val){
            $vals  =  explode("=", $val);

            $referenceNo[] = $vals[0];
            $examType[] = $vals[4];
            $observerID[] = $vals[3];
            $role[] =  $vals[1];
            $isNewBillType = $vals[2];
            $centers[] = $vals[5];
        }

        if($isNewBillType[0] == 0){
            if($role[0] == 14){
                return $anotherController->csOldReport($referenceNo,$examType ,$observerID, 'pdf');
            }elseif($role[0] == 1){
                return $anotherController->ccOldReport($referenceNo,$examType ,$observerID, 'pdf');
            }else{
                return $anotherNewController->OBS_PDF_Report($referenceNo,$examType,$role, 'pdf',$observerID,$centers);
            }

        }else{
            if($role[0] == 14){
                return $anotherNewController->CS_PDF_Report($referenceNo,$examType ,$role, 'pdf',$observerID,$centers);
            }elseif($role[0] == 1){
                return $anotherNewController->downloadPdf($referenceNo,$examType ,$role, 'pdf',$observerID,$centers);
            }else{
                return $anotherNewController->OBS_PDF_Report($referenceNo,$examType,$role, 'pdf',$observerID,$centers);
            }
        }
    }

 
    public function detailReport(Request $request)
    {
        try 
        {
            if ($request->isMethod('post')) {
                $ExamType = $request->post('ExamType');
                $Status = $request->post('Status');
                $selectedBills = $request->post('selectedBills');
                $isSelectAllChecked = $request->post('selectAllCheck');
                $Table = config('constants.CityCoordinatorOfflineTbl');
                $ObserverDetailsTable = config('constants.ObserverDetailsTable');
                $TableCRH = config('constants.CentralizedRemunerationHeaderTable');
                $TableCRL = config('constants.CentralizedRemunerationLineTableShiftWise');

            
                $checkReference = '';
                if(count($selectedBills) > 0 && $isSelectAllChecked == 0){
                    $quotedString = "'" . implode("','", $selectedBills) . "'";
                    $checkReference =  " AND [Reference No_] IN ($quotedString)";
                }
               
                
                $UnifiedData = [];
                if ($ExamType) {
                    $TableData = DB::select(
                        "exec getCommonDataSP :select, :input, :table, :orderBy",
                        [
                            'select' => "[Reference No_],[Observer ID],[Observer Name],[Mobile No_],[PAN No_],
                                        [No_ of Duty Days],[Total Deduction Amount],TDS_Amount,[Net Payable After Deduction],
                                        [City Coor_ Remuneration Amount],[City Coor_ Conveyance Amount],
                                        [City Coor_ MockDrill Remuneration],[City Coor_ MockDrill Conveyance],
                                        [Refreshment Amount],[Advance payment],[Net Amount Payable],
                                        [Miscellaneous Charge],[Rejection Remark],[Remark],[Observer TDS],[Deputy Observer TDS],[Total Expenditure Amount]",
                            'input' => "WHERE [Exam Type] = '$ExamType' AND [Bill Status] = '$Status' $checkReference",
                            'table' => "[$Table]",
                            'orderBy' => 'ORDER BY [Observer ID] ASC'
                        ]
                    );

                    if (!empty($TableData)) {
                        $RefNoArray = collect($TableData)->pluck('Reference No_')->toArray();

                        if (!empty($RefNoArray)) {
                            $ObserverDyObserverData = DB::table($ObserverDetailsTable)
                                ->whereIn('Reference_No', $RefNoArray)
                                ->get();
                                // prd($ObserverDyObserverData);

                            $ObserverMapping = [];
                            foreach ($ObserverDyObserverData as $obs) {
                                $ObserverMapping[$obs->Reference_No][] = $obs;
                            }

                            foreach ($TableData as $value) {
                                $observerId = $value->{'Observer ID'};
                                $refNo = $value->{'Reference No_'};

                                $UnifiedData[$observerId] = (array)$value;
                                $UnifiedData[$observerId]['Observer'] = [];
                                $UnifiedData[$observerId]['Deputy-Observer'] = [];

                                if (isset($ObserverMapping[$refNo])) {
                                    foreach ($ObserverMapping[$refNo] as $obs) {
                                        if ($obs->Observer_Type == 5) {
                                            $UnifiedData[$observerId]['Observer'][] = $obs;
                                        } elseif ($obs->Observer_Type == 2) {
                                            $UnifiedData[$observerId]['Deputy-Observer'][] = $obs;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                if (!empty($UnifiedData)) {
                    $CCdataArray = [];
                // prd($UnifiedData);
                    foreach ($UnifiedData as $refKey => $excelval) {
                        $totalRemuneration = 0;
                        $totalConveyance = 0;
                        $totalTDS = 0;
                        $totalExpenditure = 0;
                        $totalAdvance = 0;
                
                        // Add the main row from $UnifiedData
                        $CCdataArray[] = [
                            $excelval['Observer ID'],
                            $excelval['Observer Name'],
                            '-',
                            $excelval['Mobile No_'],
                            $excelval['PAN No_'],
                            $excelval['No_ of Duty Days'],
                            1,
                            $excelval['City Coor_ Remuneration Amount'],
                            $excelval['City Coor_ MockDrill Remuneration'],
                            $excelval['City Coor_ Conveyance Amount'] + $excelval['City Coor_ MockDrill Conveyance'],
                            $excelval['Refreshment Amount'],
                            $excelval['Miscellaneous Charge'],
                            $excelval['Total Deduction Amount'],
                            $excelval['TDS_Amount'],
                            $excelval['Total Expenditure Amount'] - ($excelval['TDS_Amount']),
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            $excelval['Rejection Remark'] ?: $excelval['Remark'],
                        ];

                        $totalTDS += $excelval['TDS_Amount'];
                        $totalExpenditure +=  $excelval['Total Expenditure Amount'] - ($excelval['TDS_Amount']);
                        $totalAdvance +=  $excelval['Advance payment'];
                        $totalConveyance = $excelval['City Coor_ Conveyance Amount'] + $excelval['City Coor_ MockDrill Conveyance'];
                
                        // Add Observers
                        if (!empty($excelval['Observer'])) {
                            foreach ($excelval['Observer'] as $observer) {
                                $CCdataArray[] = [
                                    $observer->Observer_ID,
                                    '-',
                                    $observer->Observer_Name,
                                    $observer->Observer_Phone,
                                    $observer->PAN,
                                    $observer->Duty_Days,
                                    $observer->{'Mock Duty Days'},
                                    $observer->{'Observer Remuneration'},
                                    $observer->{'Observer Mock Remuneration'},
                                    $observer->{'Observer Conveyance'} + $observer->{'Observer Mock Conveyance'},
                                    $observer->{'Observer Foods'},
                                    '-',
                                    '',
                                    $observer->{'Observer TDS'},
                                    ($observer->{'Amount To Paid'} - $observer->{'Advance_Payment'}),
                                    0, // Placeholder values
                                    0,
                                    0,
                                    0,
                                    0,
                                    0,
                                    '',
                                    ''
                                ];
                
                                // Update Totals
                                $totalRemuneration += $observer->{'Observer Remuneration'};
                                $totalConveyance += ($observer->{'Observer Conveyance'} + $observer->{'Observer Mock Conveyance'});
                                $totalTDS += $observer->{'Observer TDS'};
                                $totalExpenditure += ($observer->{'Amount To Paid'} - $observer->{'Advance_Payment'});
                                $totalAdvance += $observer->{'Advance_Payment'};
                            }
                        }
                
                        // Add Deputy Observers
                        if (!empty($excelval['Deputy-Observer'])) {
                            foreach ($excelval['Deputy-Observer'] as $deputyObserver) {
                                $CCdataArray[] = [
                                    $deputyObserver->Observer_ID,
                                    '-',
                                    $deputyObserver->Observer_Name,
                                    $deputyObserver->Observer_Phone,
                                    $deputyObserver->PAN,
                                    $deputyObserver->Duty_Days,
                                    $deputyObserver->{'Mock Duty Days'},
                                    $deputyObserver->{'Observer Remuneration'},
                                    $deputyObserver->{'Observer Mock Remuneration'},
                                    $deputyObserver->{'Observer Conveyance'} + $deputyObserver->{'Observer Mock Conveyance'},
                                    $deputyObserver->{'Observer Foods'},
                                    '-',
                                    '',
                                    $deputyObserver->{'Observer TDS'},
                                    ($deputyObserver->{'Amount To Paid'} - $deputyObserver->{'Advance_Payment'}),
                                    0, // Placeholder values
                                    0,
                                    0,
                                    0,
                                    0,
                                    0,
                                    '',
                                    ''
                                ];
                
                                // Update Totals
                                $totalRemuneration += $deputyObserver->{'Observer Remuneration'};
                                $totalConveyance += ($observer->{'Observer Conveyance'} + $observer->{'Observer Mock Conveyance'});
                                $totalTDS += $deputyObserver->{'Observer TDS'};
                                $totalExpenditure += ($deputyObserver->{'Amount To Paid'} - $deputyObserver->{'Advance_Payment'});
                                $totalAdvance += $deputyObserver->{'Advance_Payment'};
                            }
                        }
                
                        // Add Total Row
                        $CCdataArray[] = [
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            '',
                            'Total',
                            $totalTDS,
                            0, // Total MockDrill Remuneration (if applicable)
                            $totalExpenditure,
                            $totalAdvance,
                            ($totalExpenditure + $totalTDS),
                            //($totalExpenditure + $totalTDS) - ($totalAdvance + $totalTDS),
                            $excelval['Net Payable After Deduction'],
                            '',
                            ''
                        ];
                    }
                
                    // Store the Excel file
                    if (!empty($CCdataArray)) {
                        $dataCollection = new Collection($CCdataArray);
                        $fileName = 'ObserverDetailReport.xlsx';
                    
                        // Delete the file before writing, to avoid overwrite issues
                        if (Storage::disk('public')->exists($fileName)) {
                            Storage::disk('public')->delete($fileName);
                        }
                    
                        Excel::store(new ObserverDetailReportsExport($dataCollection), $fileName, 'public');
                    
                        // Add a random query string to prevent browser caching
                        $fileUrl = asset("storage/$fileName");
                    
                        // For Local;
                        return response()->json(['file' => $fileUrl]);
                        // for staging or Live
                        //return response()->json(['file' => '../../storage/app/public/'.$fileName]);
                    }
                }
                

                return response()->json(['error' => 'No data found for the given criteria'], 404);
            }
        } catch (\Exception $e) {
            return response()->json(['error' => 'Query execution failed: ' . $e->getMessage()], 500);
        }
    }


    public function expotReportDownload(Request $request)
    {
        try 
        {
            if ($request->isMethod('post')) {
              
                    $label = $request->post('status');
                    $type = $request->post('type');
                    $examType = $request->post('examType');
                   

                      // Set status and condition based on the input status
                    $status = '';
                    $THERCONDITION = '';
                    if ($label == 1) {
                        $status = "AND CCTbl.[Bill Status] IN ('1','2','4')";
                    } elseif ($label == 'UN') {
                        $THERCONDITION = "AND EX.[Submitted Documents] = '0'";
                        $status = '';
                    } elseif ($label == 'TOT') {
                        $status = '';
                        $THERCONDITION = '';
                    } else {
                        $status = 'AND CCTbl.[Bill Status] IN (' . $label . ')';
                    }


                    $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
                    $TABillHeaderTbl = config('constants.TABillHeaderTbl');
                    $CentreBillTbl = config('constants.CentreBillTbl');
                    $ExamProfile = config('constants.ExamProfile');
                    $VendorBankTbl = config('constants.VendorBankTbl');
                    $VendorPanTbl = config('constants.VendorPanTbl');
                    $VendorContactTbl = config('constants.VendorContactTbl');
                    $TABillLocalTbl = config('constants.TABillLocalTbl');
                    $TABillHotelTbl = config('constants.TABillHotelTbl');
                    $TABillOutstationTbl = config('constants.TABillOutstationTbl');
                    $examTable = config('constants.ExamDates');
                    $CandidateCountTable=config('constants.CandidateCountTable');

                    // Clean table names by removing square brackets
                    $CityCoordinatorOfflineTbl = '[' . str_replace(['[', ']'], '', $CityCoordinatorOfflineTbl) . ']';
                    $TABillHeaderTbl = '[' . str_replace(['[', ']'], '', $TABillHeaderTbl) . ']';
                    $CentreBillTbl = '[' . str_replace(['[', ']'], '', $CentreBillTbl) . ']';
                    $VendorBankTbl = '[' . str_replace(['[', ']'], '', $VendorBankTbl) . ']';
                    $VendorPanTbl = '[' . str_replace(['[', ']'], '', $VendorPanTbl) . ']';
                    $VendorContactTbl = '[' . str_replace(['[', ']'], '', $VendorContactTbl) . ']';
                    $ExamProfile = '[' . str_replace(['[', ']'], '', $ExamProfile) . ']';
                    $TABillLocalTbl = '[' . str_replace(['[', ']'], '', $TABillLocalTbl) . ']';
                    $TABillHotelTbl = '[' . str_replace(['[', ']'], '', $TABillHotelTbl) . ']';
                    $TABillOutstationTbl = '[' . str_replace(['[', ']'], '', $TABillOutstationTbl) . ']';
                    $examTable = '[' . str_replace(['[', ']'], '', $examTable) . ']';
                    $CandidateCountTable = '[' . str_replace(['[', ']'], '', $CandidateCountTable) . ']';

                    $SELECT = '';
                    $WHERE = '';
                    $TABLE = '';
                    $queryType = 0;
                    switch ($type) {
                        case 'CITY':
                            if ($label == 'UN') {
                                $queryType = 1;
                                $SELECT = "Select DISTINCT '' as ReferenceNo,
                                        ObserverID as ObserverID,
                                        Name as ObserverName,
                                        ExamType as ExamID,
                                        Mobile,
                                        BillType,
                                         STRING_AGG(CENTNO, ',') AS Centno,
                                        [Observer Type] as [role],
                                        Email,
                                        '0000-00-00 00:00:00.000'  as billsubmitdate from $ExamProfile
                                        WHERE [ExamType] = '$examType' AND [Is Dummy] = 0 AND [Submitted Documents]=0  AND  [Observer Type]  in(1)
                                        GROUP BY 
                                        [ObserverID],
                                        Name,
                                        [ExamType],
                                        Mobile, 
                                        BillType,
                                        [Observer Type],
                                        Email
                                        ";
                        
                            } else if ($label == 'TOT') {
                                $queryType = 1;
                                $SELECT = "Select DISTINCT '' as ReferenceNo,
                                        ObserverID as ObserverID,
                                        Name as ObserverName,
                                        ExamType as ExamID,
                                        Mobile,
                                        BillType,
                                        STRING_AGG(CENTNO, ',') AS Centno,
                                        [Observer Type] as [role],
                                        Email,
                                        '0000-00-00 00:00:00.000'  as billsubmitdate from $ExamProfile
                                        WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(1)
                                        GROUP BY 
                                        [ObserverID],
                                        Name,
                                        [ExamType],
                                        Mobile, 
                                        BillType,
                                        [Observer Type],
                                        Email
                                        ";
                                
                            } else {
                                $queryType = 1;
                                $SELECT = "select CCTbl.[Reference No_] AS ReferenceNo,
                                        CCTbl.IsNewBill AS BillType,
                                        EX.[ObserverID] AS ObserverID,
                                        EX.[Name] AS ObserverName,
                                        EX.[ExamType] AS ExamID,
                                        EX.Mobile,
                                        STRING_AGG(EX.CENTNO, ',') AS Centno,
                                        EX.[Observer Type] AS [role],
                                        EX.Email,
                                        [Created On] as billsubmitdate from $ExamProfile AS EX
                                        INNER JOIN $CityCoordinatorOfflineTbl as CCTbl ON EX.[ObserverID] = CCTbl.[Observer ID] AND EX.[ExamType] = CCTbl.[Exam Type] 
                                        WHERE EX.[ExamType] = '$examType' $status AND EX.[Is Dummy] = 0 $THERCONDITION GROUP BY 
                                        CCTbl.[Reference No_],
                                        CCTbl.IsNewBill,
                                        EX.[ObserverID],
                                        EX.[Name],
                                        EX.[ExamType],
                                        EX.Mobile, 
                                        EX.[Observer Type],
                                        EX.Email,
                                        [Created On] order by CCTbl.[Reference No_]";
                               
                            }
                            break;
                
                        case 'OBSERVER BILL':
                            if ($label == 'Unsubmitted') {
                                $queryType = 1;
                                $SELECT = "Select DISTINCT '' as ReferenceNo,
                                        ObserverID as ObserverID,
                                        Name as ObserverName,
                                        ExamType as ExamID,
                                        Mobile,
                                        BillType,
                                        STRING_AGG(CENTNO, ',') AS Centno,
                                        [Observer Type] as [role],
                                        Email,
                                        '0000-00-00 00:00:00.000'  as billsubmitdate from $ExamProfile
                                        WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(5, 2, 3, 10)
                                        GROUP BY 
                                        [ObserverID],
                                        Name,
                                        [ExamType],
                                        Mobile, 
                                        BillType,
                                        [Observer Type],
                                        Email
                                        ";
                               
                            } else if ($label == 'Total Bill') {
                                $queryType = 1;
                                $SELECT = "Select DISTINCT '' as ReferenceNo,
                                        ObserverID as ObserverID,
                                        Name as ObserverName,
                                        ExamType as ExamID,
                                        Mobile,
                                        BillType,
                                        STRING_AGG(CENTNO, ',') AS Centno,
                                        [Observer Type] as [role],
                                        Email,
                                        '0000-00-00 00:00:00.000'  as billsubmitdate from $ExamProfile
                                        WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(5, 2, 3, 10)
                                        GROUP BY 
                                        [ObserverID],
                                        Name,
                                        [ExamType],
                                        Mobile, 
                                        BillType,
                                        [Observer Type],
                                        Email
                                        ";
                               
                            } else {
                                $queryType = 1;
                                $SELECT = "Select DISTINCT
                                        CCTbl.[Reference No_] as ReferenceNo,
                                        CCTbl.IsNewBill as BillType,
                                        EX.[ObserverID] as ObserverID,
                                        EX.[Name] as ObserverName,
                                        EX.[ExamType] as ExamID,
                                        EX.Mobile,
                                        STRING_AGG(EX.CENTNO, ',') AS Centno,
                                        EX.[Observer Type] as [role], 
                                        EX.Email,
                                        [CreatedDate] as billsubmitdate from $ExamProfile AS EX
                                        INNER JOIN $TABillHeaderTbl as CCTbl ON EX.[ObserverID] = CCTbl.[Observer ID] AND EX.[ExamType] = CCTbl.[Exam Code] AND EX.[CENTNO] IN (SELECT value FROM STRING_SPLIT(CCTbl.[Centre No_], ','))
                                        WHERE EX.[ExamType] = '$examType' $status AND EX.[Is Dummy] = 0 $THERCONDITION GROUP BY 
                                        CCTbl.[Reference No_],
                                        CCTbl.IsNewBill,
                                        EX.[ObserverID],
                                        EX.[Name],
                                        EX.[ExamType],
                                        EX.Mobile, 
                                        EX.[Observer Type],
                                        EX.Email,
                                        [CreatedDate]
                                        order by [Reference No_]";
                               
                            }
                            break;
                
                        case 'CENTRE SUPERINTENDENT BILL':
                            if ($label == 'Unsubmitted') {
                                $SELECT = " DISTINCT '' as ReferenceNo,
                                        ObserverID as ObserverID,
                                        Name as ObserverName,
                                        ExamType as ExamID,
                                        Mobile,
                                        BillType,
                                        CENTNO as Centno,
                                        [Observer Type] as [role],
                                        Email,
                                        '0000-00-00 00:00:00.000'  as billsubmitdate";
                                $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0 AND [Submitted Documents]=0  AND  [Observer Type]  in(14)";
                                $TABLE = "$ExamProfile";
                            } else if ($label == 'Total Bill') {
                                $SELECT = " DISTINCT '' as ReferenceNo,
                                        ObserverID as ObserverID,
                                        Name as ObserverName,
                                        ExamType as ExamID,
                                        Mobile,
                                        BillType,
                                        CENTNO as Centno,
                                        [Observer Type] as [role],
                                        Email,
                                        '0000-00-00 00:00:00.000'  as billsubmitdate";
                                $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(14)";
                                $TABLE = "$ExamProfile";
                            } else {
                                $SELECT = 'DISTINCT
                                    CCTbl.[Reference No_] as ReferenceNo,
                                    CCTbl.IsNewBill as BillType,
                                    EX.[ObserverID] as ObserverID,
                                    EX.[Name] as ObserverName,
                                    EX.[ExamType] as ExamID,
                                    EX.Mobile,
                                    EX.CENTNO as Centno,
                                    EX.[Observer Type] as [role],
                                    EX.Email,
                                    [Created On] as billsubmitdate';
                                $WHERE = "INNER JOIN $CentreBillTbl as CCTbl ON EX.[ObserverID] = CCTbl.[Observer ID] AND EX.[ExamType] = CCTbl.[Exam Type] AND EX.[CENTNO] IN (SELECT value FROM STRING_SPLIT(CCTbl.[CenterNo], ','))
                                   WHERE EX.[ExamType] = '$examType' $status AND EX.[Is Dummy] = 0 $THERCONDITION order by CCTbl.[Reference No_]";
                                $TABLE = "$ExamProfile AS EX";
                            }
                            break;
                
                        default:
                            return response()->json(['error' => 'Invalid bill type'], 400);
                    }

                    if($queryType == 1){
                        $CSArray = DB::select(DB::raw($SELECT));
                    }else{
                        $CSArray = DB::select(DB::raw("EXEC getCommonDataSP :select, :input, :table"), [
                                        'select' => $SELECT,
                                        'input' => $WHERE,
                                        'table' => $TABLE,
                                    ]);
                    }

                    if($type == 'CITY'){

                        if ($label == 'UN') {

                            //start for excel for CITY COORDINATOR BILL => Unsubmitted 
                            if (isset($CSArray) && !empty($CSArray)) {
                                foreach ($CSArray as $key => $excelval) {

                                    $roleType = $excelval->role;
                                    $role=config('constants.RoleName.'.$roleType);
                                        $CCdataArray[] = [
                                            $excelval->ReferenceNo, 
                                            $excelval->ObserverID,
                                            $excelval->ObserverName,
                                            $excelval->ExamID,
                                            'Unsubmitted',
                                            $role,
                                            $excelval->Centno,
                                            $excelval->Email,
                                            $excelval->Mobile,
                                            ''
                                        ];
                                }
                            }

                            if (!empty($CCdataArray)) {
        
                                $dataCollection = new Collection($CCdataArray);
                                $fileName = 'CITY_COORDINATOR_UNSUBMITTED_EXPORT_REPORT.xlsx';
                                // Delete the file before writing, to avoid overwrite issues
                                if (Storage::disk('public')->exists($fileName)) {
                                    Storage::disk('public')->delete($fileName);
                                }
                                Excel::store(new CityBillUnsubmittedExportReport($dataCollection), $fileName, 'public');
                                // Add a random query string to prevent browser caching
                                $fileUrl = asset("storage/$fileName");
                                // For Local;
                                return response()->json(['file' => $fileUrl]);
                                // for staging or Live
                                //return response()->json(['file' => '../../storage/app/public/'.$fileName]);
                             }

                            //end for excel for CITY COORDINATOR BILL => Unsubmitted 

                        }  else if ($label == 'TOT') {

                            //start for excel for CITY COORDINATOR BILL => Total Bill  

                            if (isset($CSArray) && !empty($CSArray)) {
                                foreach ($CSArray as $key => $excelval) {

                                    $roleType = $excelval->role;
                                    $role=config('constants.RoleName.'.$roleType);
                                        $CCdataArray[] = [
                                            $excelval->ReferenceNo, 
                                            $excelval->ObserverID,
                                            $excelval->ObserverName,
                                            $excelval->ExamID,
                                            '-',
                                            $role,
                                            $excelval->Centno,
                                            $excelval->Email,
                                            $excelval->Mobile,
                                            ''
                                        ];
                                }
                            }

                            if (!empty($CCdataArray)) {
        
                                $dataCollection = new Collection($CCdataArray);
                                $fileName = 'CITY_COORDINATOR_TOTAL_BILL_EXPORT_REPORT.xlsx';
                                // Delete the file before writing, to avoid overwrite issues
                                if (Storage::disk('public')->exists($fileName)) {
                                    Storage::disk('public')->delete($fileName);
                                }
                                Excel::store(new CityBillUnsubmittedExportReport($dataCollection), $fileName, 'public');
                                // Add a random query string to prevent browser caching
                                $fileUrl = asset("storage/$fileName");
                                // For Local;
                                return response()->json(['file' => $fileUrl]);
                                // for staging or Live
                                //return response()->json(['file' => '../../storage/app/public/'.$fileName]);
                             }
                            //end for excel for CITY COORDINATOR BILL => Total Bill  

                        }else{
                            
                            //start for excel for CITY COORDINATOR BILL => Open, In Process Bill ,Rejected,Payment Completed  
                            if (isset($CSArray) && !empty($CSArray)) {
                                foreach ($CSArray as $key => $excelval) {
        
                                    $referenceNo = $excelval->ReferenceNo;
                                    $observerId = $excelval->ObserverID;
                                    $roleType = $excelval->role;
                                  
                                            $MaxCountData=[];
                                            $query = "select * from $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                                            $data = DB::select(DB::raw($query));
                                            //$viewLoad = 'ccfields';
        
                                            $emode = "select Exam_Mode from $examTable where [Exam Type]='$examType'";
                                            $exam_mode = DB::select(DB::raw($emode));
        
                                            $examMode=$exam_mode[0]->{'Exam_Mode'};
        
                                            if($examMode)
                                            {
                                                if($examMode=='CBT')
                                                {
                                                $Mode='Online';
                                                }elseif($examMode=='PNP')
                                                {
                                                    $Mode='Offline';
                                                }
                                            }
                                            $query = "select * from $VendorBankTbl where [No_]='$observerId'";
                                            $VendorBank = DB::select(DB::raw($query));
                                            $query = "select [P_A_N_ No_] from $VendorPanTbl where [No_]='$observerId'";
                                            $VendorPan = DB::select(DB::raw($query));
                                            $query = "select * from $VendorContactTbl where [No_]='$observerId'";
                                            $VendorContact = DB::select(DB::raw($query));
                                        
                                            $VendorBank=$VendorBank[0]??[];
                                            $VendorPan=$VendorPan[0]??[];
                                            $VendorContact=$VendorContact[0]??[];
                                            $tCount=count($data);
                                            $data=$data[0]??[];
                                            $role=config('constants.RoleName.'.$roleType);
                                            $BillStatus=$data->{'Bill Status'};
                                            $Role_ID=auth()->user()->Role_ID;
                                            $MaxCountVal=$MaxCountData[0]??[];
        
                                                        $CCdataArray[] = [
                                                            $referenceNo,
                                                            $role,
                                                            $observerId,
                                                            $VendorContact->Name,
                                                            $VendorContact->{'Mobile Phone No_'},
                                                            $data->{'Exam ID'},
                                                            $data->{'Exam Name'},
                                                            !empty($data->{'Exam Date'}) ? Carbon::parse($data->{'Exam Date'})->format('d-m-Y') : '',
                                                            config('constants.BillStatus.' . $data->{'Bill Status'}),
                                                            $VendorPan->{'P_A_N_ No_'},
                                                            $VendorBank->{'Branch'},
                                                            $VendorBank->{'Bank Name'},
                                                            $VendorBank->{'IFSC Code'},
                                                            $VendorBank->{'Account Holder Name'},
                                                            !empty($data->{'Completed Date'}) ? Carbon::parse($data->{'Completed Date'})->format('d-m-Y') : '',
                                                            $examType,
                                                            $data->{'Remark'},
                                                            $data->{'Rejection Remark'},
                                                            $data->{'No_ of Candidate Alloted'} ?? '',
                                                            $data->{'No_ of Candidate Alloted NTA'} ?? '',
                                                            $data->{'No_ of Candidate Present'} ?? '',
                                                            $data->{'No_ of Candidate Absent'} ?? '',
                                                            number_format($data->{'City Coor_ MockDrill Remuneration'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'City Coor_MockDrill Remuneration Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'City Coor_ MockDrill Conveyance'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'City Coor_ MockDrill Conveyance Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'City Coor_ Remuneration Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'City Coor_ Remuneration Amount Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'City Coor_ Conveyance Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'City Coor_ Conveyance Amount Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Admin Staff Mock Drill'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Admin_ Staff_Clerk Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Admin_ Staff_Clerk Amount Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Supporting Staff Mock Drill'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Supporting Staff Mock Drill Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Supporting Staff Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Supporting Staff Amount Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Refreshment Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Refreshment Deduction Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'WatchMan Mock Drill'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'WatchMan Mock Drill Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Watchman_Guard Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Watchman_Guard Amount Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'WaterBoy Mock Drill'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'WaterBoy Mock Drill Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'WaterMan Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'WaterMan Amount Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Miscellaneous Charge'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Miscellaneous Deduction Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Other Charges'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Mock Drill Amount Observer'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Mock Drill Amount Observer Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Observer Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Observer Amount Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Observer Conveyance1'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Mock Drill Amount DObserver'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Mock Drill Amount DObserver Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Deputy Observer Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Deputy Observer Amount Deduction'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Total Expenditure Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Advance Payment'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'TDSPer'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'TDS_Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Observer TDS'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Deputy Observer TDS'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Net Amount Payable'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Total Deduction Amount'} ?? 0, 2, '.', 0),
                                                            number_format($data->{'Net Payable After Deduction'} ?? 0, 2, '.', 0)
                                                        ];
                                    }
                                }
                            }
        
                              // Store the Excel file
                            if (!empty($CCdataArray)) {
        
                                    $dataCollection = new Collection($CCdataArray);
                                    $fileName = 'CITY_COORDINATOR_BILL_EXPORT_REPORT.xlsx';
                                    // Delete the file before writing, to avoid overwrite issues
                                    if (Storage::disk('public')->exists($fileName)) {
                                        Storage::disk('public')->delete($fileName);
                                    }
                                    Excel::store(new CityBillExportReport($dataCollection), $fileName, 'public');
                                    // Add a random query string to prevent browser caching
                                    $fileUrl = asset("storage/$fileName");
                                    // For Local;
                                    return response()->json(['file' => $fileUrl]);
                                    // for staging or Live
                                    //return response()->json(['file' => '../../storage/app/public/'.$fileName]);
                            }

                            //end for excel for CITY COORDINATOR BILL => Open, In Process Bill ,Rejected,Payment Completed  

                        }


                    

                   
                    
            }
        } catch (\Exception $e) {
            return response()->json(['error' => 'Query execution failed: ' . $e->getMessage()], 500);
        }
    }


}