<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Models\CommonSettingMasterData;
use Illuminate\Http\Request;
use Illuminate\Contracts\Session\Session as SessionSession;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Session;
use Maatwebsite\Excel\Facades\Excel;
use Maatwebsite\Excel\HeadingRowImport;
use Illuminate\Support\Facades\Validator;


class CommonSettingMasterDataController extends Controller
{
    public function index(Request $request)
    {
        $search = $request->get('search') ?? '';
        $perPage = 15;
        if ($request->perPage) {
            $perPage = $request->perPage;
        }

        $settingList  = getCommonSettingsData($perPage, $search);
        // prd($settingList);
        $title = 'Common Settings List';
        return view('admin.masters.settingList', ['settingList' => $settingList, 'title' => $title, 'RolesList' => config('constants.RoleNameCheck')]);
    }

    // for adding the Common Settings data
    public function add(Request $request)
    {
        $examsList = getExamDateList();
        $RolesList = config('constants.RoleNameCheck');
        $examArray = [];


        foreach ($examsList as $exam) {
            $examArray[$exam->{'Exam Type'}] = $exam->{'Exam Name'};
        }

        
        if ($request->isMethod('post')) {
            
            if($request->Exam_Type =='' && $request->role =='' && $request->travel_allowance =='' && $request->hotel_allowance =='' && $request->Attendance_Note =='' && $request->Miscellaneous_File_Note =='' && $request->instructions_texts ==''){
                return redirect()->route('setting-list')->with('error', 'At least one field must be filled in');
            }

            $checkExits = CommonSettingMasterData::where('Role',$request->role)->where('Exam_Type',$request->Exam_Type)->count();
            if($checkExits > 0){
                return redirect()->route('setting-list')->with('error', 'Role and exam type have already been added.');
            }else{
                $data = $request->all();
                $insert = CommonSettingMasterData::create([
                    'Exam_Type' => $data['Exam_Type'] ?? '',
                    'Role' => $data['role'] ?? '',
                    'Travel_Allowance' => $data['travel_allowance'] ?? '',
                    'Hotel_Allowance' => $data['hotel_allowance'] ?? '',
                    'Attendance_Note' => $data['Attendance_Note'] ?? '',
                    'Miscellaneous_File_Note' => $data['Miscellaneous_File_Note'] ?? '',
                    'Is_Active' => 1,
                    'instructions_texts' => $data['instructions_texts'] ?? '',
                ]);
                return redirect()->route('setting-list')->with('success', 'Common setting added successfully!');
            }
            
        }

        $title = 'Add Common Settings Details';

        return view('admin.masters.addEditCommonSettings', [
            'settingData' => [],
            'examsList' => $examArray,
            'title' => $title,
            'RolesList' => $RolesList,
        ]);
    }


    // for editing the Common Settings data
    public function edit(Request $request, $ID)
    {   
        $title = 'Edit Common Settings Details';
        $examsList = getExamDateList();
        $examArray = [];

        foreach ($examsList as $exam) {
            $examArray[$exam->{'Exam Type'}] = $exam->{'Exam Name'};
        }

        // Fetch the data using Eloquent
        $settingData = CommonSettingMasterData::where('id', $ID)->firstOrFail();
        // prd($settingData['travel_allowance']);
        if ($request->isMethod('post')) {
            $data = $request->all();
            $settingData->update([
                'Exam_Type' => $data['Exam_Type'] ?? '',
                'Role' => $data['role'] ?? '',
                'Travel_Allowance' => $data['travel_allowance'] ?? '',
                'Hotel_Allowance' => $data['hotel_allowance'] ?? '',
                'Attendance_Note' => $data['Attendance_Note'] ?? '',
                'Miscellaneous_File_Note' => $data['Miscellaneous_File_Note'] ?? '',
                'instructions_texts' => $data['instructions_texts'] ?? '',
                // 'Is_Active' => $data['is_active'] ?? '',
            ]);
            return redirect()->route('setting-list')->with('success', 'Common setting updated successfully!');
        }

        return view('admin.masters.addEditCommonSettings', [
            'settingData' => $settingData,
            'title' => $title,
            'examsList' => $examArray,
            'RolesList' => config('constants.RoleNameCheck'),
        ]);
    }
}
