<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Models\AdvancePayment;
use Illuminate\Http\Request;
use Illuminate\Contracts\Session\Session as SessionSession;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Session;
use Maatwebsite\Excel\Facades\Excel;
use Maatwebsite\Excel\HeadingRowImport;
use Illuminate\Support\Facades\Validator;


class ImportAdvanceDataController extends Controller
{
    // for listing the advance payment data
    public function index(Request $request)
    {
        $search = $request->get('search') ?? '';
        $perPage = 15;
        if ($request->perPage) {
            $perPage = $request->perPage;
        }

        $advanceList  = getAllAdvanceAmountData($perPage, $search);
        // prd($advanceList);
        $title = 'Advance Payment List';
        return view('admin.masters.advanceList', ['advanceList' => $advanceList, 'title' => $title]);
    }

    // for adding the advance payment data
    public function add()
    {
        $examsList = getExamDateList();
        $examArray = [];
        foreach ($examsList as $exam) {
            $examArray[$exam->{'Exam Type'}] = $exam->{'Exam Name'};
        }
        $title = 'Add Advance Payment Details';
        return view('admin.masters.addEditAdvanceDetails', ['advanceData' => [], 'examsList' => $examArray, 'title' => $title]);
    }

    // for editing the advance payment data
    public function edit(Request $request, $ID)
    {
        $title = 'Edit Advance Payment Details';
        $examsList = getExamDateList();
        $examArray = [];
        foreach ($examsList as $exam) {
            $examArray[$exam->{'Exam Type'}] = $exam->{'Exam Name'};
        }
        // Fetch the data using Eloquent
        $advanceData = AdvancePayment::where('Advance No_', $ID)->first();

        return view('admin.masters.addEditAdvanceDetails', [
            'advanceData' => $advanceData,
            'title' => $title,
            'examsList' => $examArray
        ]);
    }

    // for createint & updating the advance payment data
    public function store(Request $request, $id = null)
    {

        // Check if Advance No_ is provided (for updating) or if it's a new record (for insertion)
        $advanceNo = $request->input('Advance#No_');
        // Validate the request data
        $validated = $request->validate([
            'NTA_Observer_ID' => 'required',
            'Observer_Name' => 'required',
            'Advance_Amount' => 'required|numeric',
            'Exam_Type' => !$advanceNo ? 'required' : '',
            //'Advance#No_' => 'required',  // Add validation for Advance No_
        ]);

        if ($advanceNo) {
            // If Advance No_ is provided, try to find the record
            $advancePayment = AdvancePayment::where('Advance No_', $advanceNo)->first();

            if ($advancePayment) {
                // If the record exists, update the fields
                AdvancePayment::where('Advance No_', $advanceNo)->update([
                    'Observer Name' => $request->input('Observer_Name'),
                    'Advance Amount' => $request->input('Advance_Amount'),
                ]);
                return redirect()->route('list-advance')->with('success', 'Advance details updated successfully.');
            } else {
                return redirect()->route('list-advance')->with('error', 'Record not found.');
            }
        } else {

            $advancePayment = AdvancePayment::where('NTA Observer ID', $request->input('NTA_Observer_ID'))->where('Exam Type',$request->input('Exam_Type'))->count();
            if($advancePayment>0){
                return redirect()->route('list-advance')->with('error', 'Advance already taken.');    
            }else{
                AdvancePayment::create([
                    'Advance No_' => 'ADV-' . rand(1000, 9999), //$request->input('NTA Observer ID'),
                    'NTA Observer ID' => $request->input('NTA_Observer_ID'),
                    'Observer Name' => $request->input('Observer_Name'),
                    'Advance Amount' => $request->input('Advance_Amount'),
                    'Exam Type' => $request->input('Exam_Type'),
                ]);
                return redirect()->route('list-advance')->with('success', 'Advance details saved successfully.');    
            }
            // If Advance No_ is not provided, create a new record
        }
    }

    // for importing the advance payment data
    public function import(Request $request)
    {
        $title = 'Import Advance Payment Data';
        $examsList = getExamDateList();
        return view('admin.masters.importAdvance', ['advanceData' => [], 'examsList' => $examsList, 'title' => $title]);
    }

    public function saveimport(Request $request)
    {
        // Validate file upload
        $validator = Validator::make($request->all(), [
            'file' => 'required|mimes:xlsx,xls', // Valid file extensions
        ]);

        if ($validator->fails()) {
            return back()->withErrors($validator)->withInput();
        }

        try {
            $file = $request->file('file');
            $examType = $request->input('exam_type'); // Use input() to get the 'exam_type' value directly

            // Get the first row as headers to validate column names
            $headings = (new HeadingRowImport)->toArray($file);
            $expectedHeaders = [
                'nta_id',
                'functionary_name',
                'advance_amount',
                'bank_name',
                'account_no',
                'ifsc_code',
                'center_no'
            ];

            // Check if the first row exists and validate headers
            if (!isset($headings[0][0])) {
                return back()->with('error', 'The file format is invalid or the file is empty.');
            }

            // Flatten the first row of headings (just in case there are multiple sheets with nested arrays)
            $headers = array_map('trim', $headings[0][0]);


            // Check for missing or incorrect headers
            $missingHeaders = array_diff($expectedHeaders, $headers);
            if (!empty($missingHeaders)) {
                return back()->with('error', 'Missing or incorrect headers: ' . implode(', ', $missingHeaders));
            }

            // Check for extra headers that shouldn't be there
            $extraHeaders = array_diff($headers, $expectedHeaders);
            if (!empty($extraHeaders)) {
                return back()->with('error', 'Unexpected extra headers: ' . implode(', ', $extraHeaders));
            }

            // Convert file data to an array
            $data = Excel::toArray([], $file);

            // Define the database table from config
            $advanceTable = config('constants.AdvanceTable');

            // Initialize skipped rows and data issues
            $skippedRows = [];
            $invalidRows = [];
            $rowNumber = 1; // Track row number (or use an actual ID if available)

            // Loop through each sheet
            foreach ($data as $sheetIndex => $sheetData) {
                // Remove the header row from the sheet data
                array_shift($sheetData);

                // prd($sheetData);

                // Loop through the rows in the sheet
                foreach ($sheetData as $rowIndex => $row) {
                    // Get values from the row
                    $observerID = $row[0] ?? null;
                    $observerName = $row[1] ?? null;
                    $advanceAmount = $row[2] ?? null;
                    $bankName = $row[3] ?? null;
                    $accountNo = $row[4] ?? null;
                    $ifscCode = $row[5] ?? null;
                    $centerNo = $row[6] ?? null;

                    // Validate required data fields
                    if (empty($observerID) || empty($observerName) || empty($advanceAmount) || empty($bankName) || empty($accountNo) || empty($ifscCode) || empty($centerNo)) {
                        $invalidRows[] = $rowNumber;
                    }

                    // Check if record already exists
                    $exists = DB::table($advanceTable)
                        ->where('NTA Observer ID', $observerID)
                        ->where('Exam Type', $examType)
                        ->exists();

                    if (!$exists) {
                        // Insert the new record
                        DB::table($advanceTable)->insert([
                            'Advance No_' => 'ADV-' . rand(1000, 9999),
                            'NTA Observer ID' => $observerID,
                            'Observer Name' => $observerName,
                            'Advance Amount' => $advanceAmount,
                            'Bank Name' => $bankName,
                            'Bank A_C No_' => $accountNo,
                            'Bank IFSC Code' => $ifscCode,
                            'Center No_' => $centerNo,
                            'Exam Type' => $examType
                        ]);
                    } else {
                        // Store row number in skippedRows array
                        $skippedRows[] = $rowNumber;
                    }

                    // Increment row number
                    $rowNumber++;
                }
            }

            // Prepare error or success message
            $messages = [];

            // Handle invalid data rows
            if (!empty($invalidRows)) {
                $messages[] = "Invalid data in rows: " . implode(", ", $invalidRows) . ". Please check that all required fields are provided.";
            }

            // Handle skipped rows
            if (!empty($skippedRows)) {
                $messages[] = "Skipped rows (already exist): " . implode(", ", $skippedRows);
            }

            // If no errors, show success message
            if (empty($messages)) {
                $messages[] = 'Advance Payment Data Imported Successfully!';
            }

            return back()->with('success', implode(" | ", $messages));
        } catch (\Exception $e) {
            return back()->with('error', 'Error: ' . $e->getMessage());
        }
    }

    
    // public function ntausersSearch(Request $request)
    // {

    //     // print_r($request->all());
    //     // You can modify this query based on your needs.
    //     $searchTerm = $request->q;
      

    //     $advanceTable = config('constants.AdvanceTable');
    //     // Fetch the countries based on the search term
    //     $countries = DB::table($venderTableName)->where('No_', 'like', '%'.$searchTerm.'%')->get();
    //     return response()->json($countries);

    //     // // Fetch the countries based on the search term
    //     // $countries = Country::where('name', 'like', '%'.$searchTerm.'%')->get();

    //     // // Return the results in the format select2 expects
    //     // return response()->json($countries);
    // }

    public function ntausersSearch(Request $request)
    {
        $searchTerm = $request->input('q'); // Safer way to get input
       
        $venderTableName = config('constants.VendorContactTbl');
        // if (empty($searchTerm)) {
        //     return response()->json(['error' => 'Search term is required.'], 400);
        // }
        $results = DB::table($venderTableName)
            ->where('No_', 'like', '%'.$searchTerm.'%')
            ->select('No_','Name')
            ->get();
        // print_r($results);

        if ($results->isEmpty()) {
            return response()->json(['message' => 'No results found.'], 404);
        }

        // Return the results as JSON
        return response()->json($results);
    }



}
