<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Models\CandidateWiseCountData;
use Illuminate\Http\Request;
use Illuminate\Contracts\Session\Session as SessionSession;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Session;
use Maatwebsite\Excel\Facades\Excel;
use Maatwebsite\Excel\HeadingRowImport;
use Illuminate\Support\Facades\Validator;


class ImportCandidateWiseCountDataController extends Controller
{
    public function index(Request $request)
    {
        $search = $request->get('search') ?? '';
        $perPage = 15;
        if ($request->perPage) {
            $perPage = $request->perPage;
        }

        $countList  = getCandidateWiseCountData($perPage, $search);
        // prd($countList);
        $title = 'Candidate Wise Count List';
        return view('admin.masters.countList', ['countList' => $countList, 'title' => $title]);
    }

    // for adding the Candidate Wise Count data
    public function add()
    {
        $examsList = getExamDateList();
        $examArray = [];
        foreach ($examsList as $exam) {
            $examArray[$exam->{'Exam Type'}] = $exam->{'Exam Name'};
        }
        $title = 'Add Candidate Wise Count Details';
        return view('admin.masters.addEditCountDetails', ['countData' => [], 'examsList' => $examArray, 'title' => $title]);
    }

    // for editing the Candidate Wise Count data
    public function edit(Request $request, $ID)
    {
        $title = 'Edit Candidate Wise Count Details';
        $examsList = getExamDateList();
        $examArray = [];
        foreach ($examsList as $exam) {
            $examArray[$exam->{'Exam Type'}] = $exam->{'Exam Name'};
        }
        // Fetch the data using Eloquent
        $countData = CandidateWiseCountData::where('ID', $ID)->first();
        // prd($countData);
        // Pass the data to the view
        return view('admin.masters.addEditCountDetails', [
            'countData' => $countData,
            'title' => $title,
            'examsList' => $examArray
        ]);
    }

    // for createint & updating the Candidate Wise Count data
    public function store(Request $request)
    {
        
        $CandidateCountTable = config('constants.CandidateCountTable');
        $validator = Validator::make($request->all(), [
            'uploadFilesCandidateWiseCount' => 'required|mimes:xlsx,xls', // Valid file extensions
            'Exam_Type' => 'required', // Valid file extensions
        ]);
        if ($validator->fails()) {
            return back()->withErrors($validator)->withInput();
        }
        $file = $request->file('uploadFilesCandidateWiseCount');

        // Get the first row as headers to validate column names
        $headings = (new HeadingRowImport)->toArray($file);
        //$expectedHeaders = ['Exam ID/OBS_ID,CCID,SCID', 'exam_type', 'exam_name', 'exam_mode', 'year', 'examdisp', 'is_examover'];

        $expectedHeaders = [
            'CenterNo',
            'D1S1',
            'D1S2',
            'D1S3',
            'D1S4',
            'D2S1',
            'D2S2',
            'D2S3',
            'D2S4',
            'D3S1',
            'D3S2',
            'D3S3',
            'D3S4',
            'D4S1',
            'D4S2',
            'D4S3',
            'D4S4',
            'D5S1',
            'D5S2',
            'D5S3',
            'D5S4',
            'D6S1',
            'D6S2',
            'D6S3',
            'D6S4',
            'D7S1',
            'D7S2',
            'D7S3',
            'D7S4',
            'D8S1',
            'D8S2',
            'D8S3',
            'D8S4',
            'D9S1',
            'D9S2',
            'D9S3',
            'D9S4',
            'D10S1',
            'D10S2',
            'D10S3',
            'D10S4',
            'D11S1',
            'D11S2',
            'D11S3',
            'D11S4',
            'D12S1',
            'D12S2',
            'D12S3',
            'D12S4'
        ];

        if (!isset($headings[0][0]) || !is_array($headings[0][0])) {
            return back()->withErrors(['error' => 'Error: Invalid file format. Headers missing or incorrect.']);
        }
        // Trim the input array to match the expected headers count
        $fileHeaders = array_slice($headings[0][0], 0, count($expectedHeaders));

        $expectedHeaders = array_map('trim', array_map('strtolower', $expectedHeaders));
        $fileHeaders = array_map('trim', array_map('strtolower', $fileHeaders));

        $missingHeaders = array_diff($expectedHeaders, $fileHeaders);
        // Check for extra headers
        $extraHeaders = array_diff($fileHeaders, $expectedHeaders);
        if (!empty($missingHeaders) || !empty($extraHeaders)) {
            return back()->withErrors(['error', 'Error : Selected file headers do not match.
            Missing: ' . implode(', ', $missingHeaders) .
                ' | Extra: ' . implode(', ', $extraHeaders)]);
        }

        $insertCount = 0; 
        $skipCount = 0;

        $data = Excel::toArray([], $file);
        foreach ($data as $sheetIndex => $sheetData) {
            array_shift($sheetData);
            foreach ($sheetData as $row) {
                
                $checkExitsRes = DB::table($CandidateCountTable)
                    ->where('Exam Type', $request->Exam_Type)
                    ->where('CenterNo', $row[0])
                    ->get();
                if ($checkExitsRes->isNotEmpty()) {    
                    $skipCount++; 
                }else{
                    $D1Values = array_slice($row, 1, 4); // D1S1 to D1S4
                    $D2Values = array_slice($row, 5, 4); // D2S1 to D2S4
                    $D3Values = array_slice($row, 9, 4); // D3S1 to D3S4
                    $D4Values = array_slice($row, 13, 4);
                    $D5Values = array_slice($row, 17, 4);
                    $D6Values = array_slice($row, 21, 4);
                    $D7Values = array_slice($row, 25, 4);
                    $D8Values = array_slice($row, 29, 4);
                    $D9Values = array_slice($row, 33, 4);
                    $D10Values = array_slice($row, 37, 4);
                    $D11Values = array_slice($row, 41, 4);
                    $D12Values = array_slice($row, 45, 4);

                    // Find max for each group
                    $D1MAX = max($D1Values);
                    $D2MAX = max($D2Values);
                    $D3MAX = max($D3Values);
                    $D4MAX = max($D4Values);
                    $D5MAX = max($D5Values);
                    $D6MAX = max($D6Values);
                    $D7MAX = max($D7Values);
                    $D8MAX = max($D8Values);
                    $D9MAX = max($D9Values);
                    $D10MAX = max($D10Values);
                    $D11MAX = max($D11Values);
                    $D12MAX = max($D12Values);

                    // Calculate the maximum number among the max values of each group
                    $MaxNumber = max($D1MAX, $D2MAX, $D3MAX, $D4MAX,$D5MAX,$D6MAX,$D7MAX,$D8MAX,$D9MAX,$D10MAX,$D11MAX,$D12MAX);
                    // Calculate the sum of the maximum values
                    $SumOfTotal = $D1MAX + $D2MAX + $D3MAX + $D4MAX + $D5MAX + $D6MAX + $D7MAX + $D8MAX + $D9MAX + $D10MAX + $D11MAX + $D12MAX;
                    
                    DB::table($CandidateCountTable)->insert([
                        'Exam Type' => $request->Exam_Type ?? '',
                        'CenterNo' => $row[0] ?? 0,
                        'D1S1' => $row[1] ?? 0,
                        'D1S2' => $row[2] ?? 0,
                        'D1S3' => $row[3] ?? 0,
                        'D1S4' => $row[4] ?? 0,
                        'D2S1' => $row[5] ?? 0,
                        'D2S2' => $row[6] ?? 0,
                        'D2S3' => $row[7] ?? 0,
                        'D2S4' => $row[8] ?? 0,
                        'D3S1' => $row[9] ?? 0,
                        'D3S2' => $row[10] ?? 0,
                        'D3S3' => $row[11] ?? 0,
                        'D3S4' => $row[12] ?? 0,
                        'D4S1' => $row[13] ?? 0,
                        'D4S2' => $row[14] ?? 0,
                        'D4S3' => $row[15] ?? 0,
                        'D4S4' => $row[16] ?? 0,
                        'D5S1' => $row[17] ?? 0,
                        'D5S2' => $row[18] ?? 0,
                        'D5S3' => $row[19] ?? 0,
                        'D5S4' => $row[20] ?? 0,
                        'D6S1' => $row[21] ?? 0,
                        'D6S2' => $row[22] ?? 0,
                        'D6S3' => $row[23] ?? 0,
                        'D6S4' => $row[24] ?? 0,
                        'D7S1' => $row[25] ?? 0,
                        'D7S2' => $row[26] ?? 0,
                        'D7S3' => $row[27] ?? 0,
                        'D7S4' => $row[28] ?? 0,
                        'D8S1' => $row[29] ?? 0,
                        'D8S2' => $row[30] ?? 0,
                        'D8S3' => $row[31] ?? 0,
                        'D8S4' => $row[32] ?? 0,
                        'D9S1' => $row[33] ?? 0,
                        'D9S2' => $row[34] ?? 0,
                        'D9S3' => $row[35] ?? 0,
                        'D9S4' => $row[36] ?? 0,
                        'D10S1' => $row[37] ?? 0,
                        'D10S2' => $row[38] ?? 0,
                        'D10S3' => $row[39] ?? 0,
                        'D10S4' => $row[40] ?? 0,
                        'D11S1' => $row[41] ?? 0,
                        'D11S2' => $row[42] ?? 0,
                        'D11S3' => $row[43] ?? 0,
                        'D11S4' => $row[44] ?? 0,
                        'D12S1' => $row[45] ?? 0,
                        'D12S2' => $row[46] ?? 0,
                        'D12S3' => $row[47] ?? 0,
                        'D12S4' => $row[48] ?? 0,
                        'D1Max' => $D1MAX ?? 0,
                        'D2Max' => $D2MAX ?? 0,
                        'D3Max' => $D3MAX ?? 0,
                        'D4Max' => $D4MAX ?? 0,
                        'D5Max' => $D5MAX ?? 0,
                        'D6Max' => $D6MAX ?? 0,
                        'D7Max' => $D7MAX ?? 0,
                        'D8Max' => $D8MAX ?? 0,
                        'D9Max' => $D9MAX ?? 0,
                        'D10Max' => $D10MAX ?? 0,
                        'D11Max' => $D11MAX ?? 0,
                        'D12Max' => $D12MAX ?? 0,
                        'Max Count Exam' => $MaxNumber ?? 0, 
                        'Total Candidate Count in Exam' => $SumOfTotal ?? 0
                    ]);
                    $insertCount++; 
                }
            }
        }

        if ($insertCount > 0 || $skipCount > 0) {
            $message = " Imported rows: $insertCount.  Skipped rows : $skipCount.";
            return back()->with('success', $message);
        }
      
        // $ID = $request->input('ID');
        // // Validate request data
        // $validated = $request->validate([
        //     'CenterNo' => 'required|numeric',
        //     'D1Max' => 'required|numeric',
        //     'Max_Count_Exam' => 'required|numeric',
        //     'Total_Candidate_Count_in_Exam' => 'required|numeric',
        //     'Exam_Type' => $ID ? 'nullable' : 'required', // Exam_Type is required only for new records
        // ]);

        // // Prepare data array for update or insert
        // $data = [
        //     'Max Count Exam' => $request->input('Max_Count_Exam'),
        //     'Total Candidate Count in Exam' => $request->input('Total_Candidate_Count_in_Exam'),
        //     'CenterNo' => $request->input('CenterNo'),
        //     'Exam Type' => $request->input('Exam_Type'),
        // ];

        // // Remove null values to prevent overwriting existing data with null
        // $data = array_filter($data, fn($value) => !is_null($value));
        // $a=[];
        // for ($i = 1; $i <= 12; $i++) {
        //     $field = "D{$i}Max";
                       
        //     if ($request->has($field)) {   
        //         foreach($a as  $val)
        //         {
        //         $data[$val] = 0;
        //         }  
        //         $a = [];          
        //         $data[$field] = $request->input($field);
        //     }else{
        //         $a[$i]=$field;
        //     }

        // }

        // // prd($data);
        // // prd($id);
        // // Update if ID exists, otherwise create new record
        // if ($ID) {
        //     // If ID is provided, try to find the record
        //     $CountDataOld = CandidateWiseCountData::where('ID', $ID)->first();

        //     if ($CountDataOld) {
        //         // If the record exists, update the fields
        //         CandidateWiseCountData::where('ID', $ID)->update($data);

        //         return redirect()->route('list-count')->with('success', 'Candidate Count details updated successfully.');
        //     } else {
        //         return redirect()->route('list-count')->with('error', 'Record not found.');
        //     }
        // } else {
        //     // If ID is not provided, create a new record
        //     CandidateWiseCountData::create($data);

        //     return redirect()->route('list-count')->with('success', 'Candidate Count details saved successfully.');
        // }
    }


    // for importing the Candidate Wise Count data
    public function import(Request $request)
    {
        $title = 'Import Candidate Wise Count Data';
        $examsList = getExamDateList();
        return view('admin.masters.importCount', ['countData' => [], 'examsList' => $examsList, 'title' => $title]);
    }

    public function saveimport(Request $request)
    {
        // Validate file upload
        $validator = Validator::make($request->all(), [
            'file' => 'required|mimes:xlsx,xls', // Valid file extensions
        ]);

        if ($validator->fails()) {
            return back()->withErrors($validator)->withInput();
        }

        try {
            $file = $request->file('file');
            $examType = $request->input('exam_type'); // Use input() to get the 'exam_type' value directly

            // Get the first row as headers to validate column names
            $headings = (new HeadingRowImport)->toArray($file);
            $expectedHeaders = [
                'Exam Type',
                'CenterNo',
                'D1Max',
                'D2Max',
                'D3Max',
                'D4Max',
                'D5Max',
                'D6Max',
                'D7Max',
                'D8Max',
                'D9Max',
                'D10Max',
                'D11Max',
                'D21Max',
                'Max Count Exam',
                'Total Candidate Count in Exam',
            ];

            // Check if the first row exists and validate headers
            if (!isset($headings[0][0])) {
                return back()->with('error', 'The file format is invalid or the file is empty.');
            }

            // Flatten the first row of headings (just in case there are multiple sheets with nested arrays)
            $headers = array_map('trim', $headings[0][0]);


            // Check for missing or incorrect headers
            $missingHeaders = array_diff($expectedHeaders, $headers);
            if (!empty($missingHeaders)) {
                return back()->with('error', 'Missing or incorrect headers: ' . implode(', ', $missingHeaders));
            }

            // Check for extra headers that shouldn't be there
            $extraHeaders = array_diff($headers, $expectedHeaders);
            if (!empty($extraHeaders)) {
                return back()->with('error', 'Unexpected extra headers: ' . implode(', ', $extraHeaders));
            }

            // Convert file data to an array
            $data = Excel::toArray([], $file);

            // Define the database table from config
            $advanceTable = config('constants.AdvanceTable');

            // Initialize skipped rows and data issues
            $skippedRows = [];
            $invalidRows = [];
            $rowNumber = 1; // Track row number (or use an actual ID if available)

            // Loop through each sheet
            foreach ($data as $sheetIndex => $sheetData) {
                // Remove the header row from the sheet data
                array_shift($sheetData);

                // prd($sheetData);

                // Loop through the rows in the sheet
                foreach ($sheetData as $rowIndex => $row) {
                    // Get values from the row
                    $observerID = $row[0] ?? null;
                    $observerName = $row[1] ?? null;
                    $advanceAmount = $row[2] ?? null;
                    $bankName = $row[3] ?? null;
                    $accountNo = $row[4] ?? null;
                    $ifscCode = $row[5] ?? null;
                    $centerNo = $row[6] ?? null;

                    // Validate required data fields
                    if (empty($observerID) || empty($observerName) || empty($advanceAmount) || empty($bankName) || empty($accountNo) || empty($ifscCode) || empty($centerNo)) {
                        $invalidRows[] = $rowNumber;
                    }

                    // Check if record already exists
                    $exists = DB::table($advanceTable)
                        ->where('NTA Observer ID', $observerID)
                        ->where('Exam Type', $examType)
                        ->exists();

                    if (!$exists) {
                        // Insert the new record
                        DB::table($advanceTable)->insert([
                            'Advance No_' => 'ADV-' . rand(1000, 9999),
                            'NTA Observer ID' => $observerID,
                            'Observer Name' => $observerName,
                            'Advance Amount' => $advanceAmount,
                            'Bank Name' => $bankName,
                            'Bank A_C No_' => $accountNo,
                            'Bank IFSC Code' => $ifscCode,
                            'Center No_' => $centerNo,
                            'Exam Type' => $examType
                        ]);
                    } else {
                        // Store row number in skippedRows array
                        $skippedRows[] = $rowNumber;
                    }

                    // Increment row number
                    $rowNumber++;
                }
            }

            // Prepare error or success message
            $messages = [];

            // Handle invalid data rows
            if (!empty($invalidRows)) {
                $messages[] = "Invalid data in rows: " . implode(", ", $invalidRows) . ". Please check that all required fields are provided.";
            }

            // Handle skipped rows
            if (!empty($skippedRows)) {
                $messages[] = "Skipped rows (already exist): " . implode(", ", $skippedRows);
            }

            // If no errors, show success message
            if (empty($messages)) {
                $messages[] = 'Candidate Wise Count Data Imported Successfully!';
            }

            return back()->with('success', implode(" | ", $messages));
        } catch (\Exception $e) {
            return back()->with('error', 'Error: ' . $e->getMessage());
        }
    }

    public function deleteCandidateCount(Request $request,$id){
        $CandidateCountTable = config('constants.CandidateCountTable');  
        $delete = DB::table($CandidateCountTable)->where('ID',$id)->delete();
        if ($delete) {
            return response()->json(['success' => 'Candidate wise-count deleted successfully']);
        } else {
            return response()->json(['error' => 'Failed to delete wise-count'], 400);
        }
    }
}
