<?php

namespace App\Imports;

use App\Models\Admin\Exam;
use Carbon\Carbon;
use Maatwebsite\Excel\Concerns\ToModel;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\ToCollection;
use Maatwebsite\Excel\Concerns\WithValidation;
use Maatwebsite\Excel\Concerns\WithHeadingRow;


class ExamImport implements ToCollection,  WithHeadingRow
{

    public function collection(Collection $rows)
    {
        $expectedHeaders = [
            'examname',
            'year',
            'exammode',
            'letterdate',
            'date1',
            'date2',
            'date3',
            'date4',
            'date5',
            'date6',
            'date7',
            'date8',
            'date9',
            'date10',
            'date11',
            'date12',
            // 'mock_allowed',
            'is_active'
        ];

        $filteredRow = [];
        foreach ($rows as $row) {
            $filteredRow[] = array_filter($row->toArray(), function ($value, $key) {
                return !is_numeric($key);
            }, ARRAY_FILTER_USE_BOTH);
        }

        if (empty($filteredRow)) {
            return redirect()->route('exam.index')->with('warning', 'The uploaded file is empty or not properly formatted.');
        }

        $headers = array_keys($filteredRow[0]);
        // echo "<pre>";
        // print_r($headers);
        // print_r($expectedHeaders);
        // die;
        if ($headers !== $expectedHeaders) {
            return redirect()->route('exam.index')->with('warning', 'Invalid Excel header format. Please make sure the headers are formatted as: ' . implode(', ', $expectedHeaders));
        }

        $ExamDates = config('constants.ExamDates');
        $maxExamID = DB::table(DB::raw("[$ExamDates]"))
            ->selectRaw('MAX(CASE WHEN ISNUMERIC([Exam Type Id]) = 1 THEN CAST([Exam Type Id] AS INT) ELSE NULL END) as examID')
            ->first();
        $newExamID = isset($maxExamID->examID) ? (int) $maxExamID->examID + 1 : 1;

        $existingExams = []; // Store already existing exams
        $insertedExams = []; // Store successfully inserted exams

        foreach ($rows as $row) {
            // Check if examname already exists
            $existingExam = Exam::where('Exam Name', $row['examname'])->first();

            if ($existingExam) {
                $existingExams[] = $row['examname']; // Collect already existing exams
                continue; // Skip inserting this one
            }

            $examDates = [
                $this->parseDate($row['date1']),
                $this->parseDate($row['date2']),
                $this->parseDate($row['date3']),
                $this->parseDate($row['date4']),
                $this->parseDate($row['date5']),
                $this->parseDate($row['date6']),
                $this->parseDate($row['date7']),
                $this->parseDate($row['date8']),
                $this->parseDate($row['date9']),
                $this->parseDate($row['date10']),
                $this->parseDate($row['date11']),
                $this->parseDate($row['date12']),
            ];

            // Filter out null or empty values
            $validDates = array_filter($examDates);

            // Find min and max dates
            $minDate = !empty($validDates) ? min($validDates)  : null;
            $maxDate = !empty($validDates) ? max($validDates)  : null;

            // Format Exam Duration
            $examDuration = ($minDate && $maxDate)
                ? Carbon::parse($minDate)->format('d-m-Y') . " To " . Carbon::parse($maxDate)->format('d-m-Y')
                : null;

            // Set Mock Date (One day before minDate)
            $mockDate = $minDate ? Carbon::parse($minDate)->subDay()->format('Y-m-d') : null;


            // Insert new exam
            Exam::create([
                // 'Exam Type Id' => $newExamID++,
                'Exam Name' => $row['examname'],
                'Exam Type' => str_replace(' ', '', $row['examname']) . $row['year'],
                'Year' => $row['year'],
                'ExamDisp' => $row['examname'] . ' ' . $row['year'],
                'Exam_Mode' => $row['exammode'],
                'Exam_Duration' => $examDuration,
                'Letter_Date' => !empty($row['letterdate']) && strtotime($row['letterdate']) !== false
                    ? Carbon::parse($row['letterdate'])->format('Y-m-d')
                    : null,
                'Mock_Date' => $mockDate,
                'Date_1' => $this->parseDate($row['date1']),
                'Date_2' => $this->parseDate($row['date2']),
                'Date_3' => $this->parseDate($row['date3']),
                'Date_4' => $this->parseDate($row['date4']),
                'Date_5' => $this->parseDate($row['date5']),
                'Date_6' => $this->parseDate($row['date6']),
                'Date_7' => $this->parseDate($row['date7']),
                'Date_8' => $this->parseDate($row['date8']),
                'Date_9' => $this->parseDate($row['date9']),
                'Date_10' => $this->parseDate($row['date10']),
                'Date_11' => $this->parseDate($row['date11']),
                'Date_12' => $this->parseDate($row['date12']),
                'Mock_Allowed' => 1,
                'Is_Active' => $row['is_active'],
            ]);

            $insertedExams[] = $row['examname']; // Collect inserted exams
        }

        $message = '';
        $successMessage = '';
        $warningMessage = '';

        if (!empty($insertedExams)) {
            $successMessage = "Successfully inserted exams: " . implode(', ', $insertedExams) . ". ";
        }

        if (!empty($existingExams)) {
            $warningMessage = "The following exams already exist and were not inserted: " . implode(', ', $existingExams) . ".";
        }

        if (!empty($successMessage) || !empty($warningMessage)) {
            return redirect()->route('exam.index')
                ->with('success', $successMessage)
                ->with('warning', $warningMessage);
        }
    }

    private function parseDate($date)
    {
        return !empty($date) && strtotime($date) !== false
            ? Carbon::parse($date)->format('Y-m-d')
            : null;
    }
}
